@@ -60,7 +60,6 @@
 #include "common/eventlog.h"
 #include "common/packet.h"
 #include "common/bnet_protocol.h"
-#include "common/w3xp_protocol.h"
 #include "common/tag.h"
 #include "common/util.h"
 #include "common/bnettime.h"
@@ -68,6 +67,7 @@
 #include "common/list.h"
 #include "common/proginfo.h"
 #include "common/bn_type.h"
+#include "common/xalloc.h"
 #include "connection.h"
 #include "anongame.h"
 #include "prefs.h"
@@ -78,6 +78,8 @@
 #include "channel.h"
 #include "anongame.h"
 #include "storage.h"
+#include "server.h"
+#include "compat/uint.h"
 #include "common/setup_after.h"
 
 static t_list *clanlist_head = NULL;
@@ -120,10 +122,28 @@ extern int clan_send_packet_to_online_members(t_clan * clan, t_packet * packet)
 
     LIST_TRAVERSE(clan->members, curr)
     {
-	t_clanmember *member;
-	char const *clienttag;
-	if ((member = elem_get_data(curr)) && (member->memberconn) && (clienttag = conn_get_clienttag(member->memberconn)) && ((strcasecmp(clienttag, CLIENTTAG_WARCRAFT3) == 0) || (strcasecmp(clienttag, CLIENTTAG_WAR3XP) == 0)))
-	    conn_push_outqueue(member->memberconn, packet);
+	t_clanmember *	member;
+	t_clienttag	clienttag;
+	t_connection *	conn;
+	
+	if (!(member = elem_get_data(curr)))
+	{
+		eventlog(eventlog_level_error,__FUNCTION__,"got NULL elem in list");
+		continue;
+	}
+	if (!(conn = clanmember_get_conn(member)))
+		continue;				// not online
+		
+	if (!(clienttag = conn_get_clienttag(conn)))
+	{
+		eventlog(eventlog_level_error,__FUNCTION__,"conn has NULL clienttag");
+		continue;
+	}
+
+	if ((clienttag !=  CLIENTTAG_WARCRAFT3_UINT ) && (clienttag != CLIENTTAG_WAR3XP_UINT))
+		continue;				// online but wrong client
+		
+	conn_push_outqueue(conn, packet);
     }
 
     return 0;
@@ -134,37 +154,59 @@ extern int clan_send_status_window_on_create(t_clan * clan)
     t_packet * rpacket;
     t_elem *curr;
 
-    if (clan == NULL)
+    if (!(clan))
     {
 	eventlog(eventlog_level_error, __FUNCTION__, "got NULL clan");
 	return -1;
     }
 
-	if ((rpacket = packet_create(packet_class_bnet)) != NULL)
+    if ((rpacket = packet_create(packet_class_bnet)))
     {
 	char channelname[10];
 	if (clan->clantag)
 	    sprintf(channelname, "Clan %c%c%c%c", (clan->clantag >> 24), (clan->clantag >> 16) & 0xff, (clan->clantag >> 8) & 0xff, clan->clantag & 0xff);
+	else
+	{
+	    sprintf(channelname, "Clans");
+	    eventlog(eventlog_level_error,__FUNCTION__,"clan has NULL clantag");
+	}
+	    
 	packet_set_size(rpacket, sizeof(t_server_w3xp_clan_clanack));
 	packet_set_type(rpacket, SERVER_W3XP_CLAN_CLANACK);
 	bn_byte_set(&rpacket->u.server_w3xp_clan_clanack.unknow1, 0);
 	bn_int_set(&rpacket->u.server_w3xp_clan_clanack.clantag, clan->clantag);
+	
 	LIST_TRAVERSE(clan->members, curr)
 	{
-	    t_clanmember *member;
-	    char const *clienttag;
-	    if ((member = elem_get_data(curr)) && member->memberconn && (clienttag = conn_get_clienttag(member->memberconn)) && ((strcasecmp(clienttag, CLIENTTAG_WARCRAFT3) == 0) || (strcasecmp(clienttag, CLIENTTAG_WAR3XP) == 0)))
+	    t_clanmember *	member;
+	    t_clienttag 	clienttag;
+	    t_connection *	conn;
+	    
+	    if (!(member = elem_get_data(curr)))
 	    {
-		if (conn_get_channel(member->memberconn))
-		{
-		    conn_update_w3_playerinfo(member->memberconn);
-		    channel_set_flags(member->memberconn);
-		    if (conn_set_channel(member->memberconn, channelname) < 0)
-			conn_set_channel(member->memberconn, CHANNEL_NAME_BANNED);	/* should not fail */
-		}
-		bn_byte_set(&rpacket->u.server_w3xp_clan_clanack.status, member->status);
-		conn_push_outqueue(member->memberconn, rpacket);
+	    	eventlog(eventlog_level_error,__FUNCTION__,"got NULL elem in list");
+		continue;
 	    }
+	    if (!(conn = clanmember_get_conn(member)))
+	    	continue;			// not online;
+		
+	    if (!(clienttag = conn_get_clienttag(conn)))
+	    {
+		eventlog(eventlog_level_error,__FUNCTION__,"conn has NULL clienttag");
+		continue;
+	    }
+	    if ((clienttag != CLIENTTAG_WARCRAFT3_UINT) && (clienttag != CLIENTTAG_WAR3XP_UINT))
+	    	continue;			// online but wrong client
+		
+	    if (conn_get_channel(conn))
+	    {
+	        conn_update_w3_playerinfo(conn);
+	        channel_set_userflags(conn);
+	        if (conn_set_channel(conn, channelname) < 0)
+		    conn_set_channel(conn, CHANNEL_NAME_BANNED);	/* should not fail */
+	    }
+	    bn_byte_set(&rpacket->u.server_w3xp_clan_clanack.status, member->status);
+	    conn_push_outqueue(conn, rpacket);
 	}
 	packet_del_ref(rpacket);
     }
@@ -176,26 +218,42 @@ extern int clan_close_status_window_on_disband(t_clan * clan)
     t_packet * rpacket;
     t_elem *curr;
 
-    if (clan == NULL)
+    if (!(clan))
     {
 	eventlog(eventlog_level_error, __FUNCTION__, "got NULL clan");
 	return -1;
     }
 
-	if ((rpacket = packet_create(packet_class_bnet)) != NULL)
+    if ((rpacket = packet_create(packet_class_bnet)))
     {
-	packet_set_size(rpacket, sizeof(t_server_w3xp_clan_clanleaveack));
-	packet_set_type(rpacket, SERVER_W3XP_CLAN_CLANLEAVEACK);
-	bn_byte_set(&rpacket->u.server_w3xp_clan_clanack.unknow1, SERVER_W3XP_CLAN_CLANLEAVEACK_UNKNOWN1);
+	packet_set_size(rpacket, sizeof(t_server_w3xp_clanquitnotify));
+	packet_set_type(rpacket, SERVER_W3XP_CLANQUITNOTIFY);
+	bn_byte_set(&rpacket->u.server_w3xp_clan_clanack.status, SERVER_W3XP_CLANQUITNOTIFY_STATUS_REMOVED_FROM_CLAN);
 	LIST_TRAVERSE(clan->members, curr)
 	{
-	    t_clanmember *member;
-	    char const *clienttag;
-	    if ((member = elem_get_data(curr)) && member->memberconn && (clienttag = conn_get_clienttag(member->memberconn)) && ((strcasecmp(clienttag, CLIENTTAG_WARCRAFT3) == 0) || (strcasecmp(clienttag, CLIENTTAG_WAR3XP) == 0)))
+	    t_clanmember *	member;
+	    t_clienttag 	clienttag;
+	    t_connection *	conn;
+	    
+	    if (!(member = elem_get_data(curr)))
 	    {
-		conn_push_outqueue(member->memberconn, rpacket);
-		conn_update_w3_playerinfo(member->memberconn);
+	    	eventlog(eventlog_level_error,__FUNCTION__,"got NULL elem in list");
+		continue;
 	    }
+	    if (!(conn = clanmember_get_conn(member)))
+	    	continue;			// not online;
+		
+	    if (!(clienttag = conn_get_clienttag(conn)))
+	    {
+		eventlog(eventlog_level_error,__FUNCTION__,"conn has NULL clienttag");
+		continue;
+	    }
+
+	    if ((clienttag != CLIENTTAG_WARCRAFT3_UINT) && (clienttag != CLIENTTAG_WAR3XP_UINT))
+	    	continue;			// online but wrong client
+		
+	    conn_push_outqueue(conn, rpacket);
+	    conn_update_w3_playerinfo(conn);
 	}
 	packet_del_ref(rpacket);
     }
@@ -207,17 +265,40 @@ extern int clan_close_status_window_on_disband(t_clan * clan)
 extern int clan_send_status_window(t_connection * c)
 {
     t_packet * rpacket;
-    t_account *acc = conn_get_account(c);
+    t_account *acc;
     t_clanmember *member;
-    char const *clienttag;
+    t_clienttag clienttag;
+    t_clan * clan;
 
-    if (acc == NULL)
+    if (!(acc = conn_get_account(c)))
 	return 0;
 
-    if ((member = account_get_clanmember(acc)) && member->clan && (member->clan->clantag) && (clienttag = conn_get_clienttag(c)) && ((strcasecmp(clienttag, CLIENTTAG_WARCRAFT3) == 0) || (strcasecmp(clienttag, CLIENTTAG_WAR3XP) == 0)))
+    if (!(member = account_get_clanmember(acc)))
+    	return 0;
+
+    if (!(clan = member->clan))
+    {
+	eventlog(eventlog_level_error,__FUNCTION__,"member has NULL clan");
+	return -1;
+    }
+
+    if (!(clan->clantag))
+    {
+	eventlog(eventlog_level_error,__FUNCTION__,"clan has NULL clantag");
+	return -1;
+    }
+    
+    if (!(clienttag = conn_get_clienttag(c)))
+    {
+	eventlog(eventlog_level_error,__FUNCTION__,"conn has NULL clienttag");
+	return -1;
+    }
+    
+    if ((clienttag != CLIENTTAG_WARCRAFT3_UINT) && (clienttag != CLIENTTAG_WAR3XP_UINT))
+    	return 0;
+	
+    if ((rpacket = packet_create(packet_class_bnet)))
     {
-	if ((rpacket = packet_create(packet_class_bnet)) != NULL)
-	{
 	    packet_set_size(rpacket, sizeof(t_server_w3xp_clan_clanack));
 	    packet_set_type(rpacket, SERVER_W3XP_CLAN_CLANACK);
 	    bn_byte_set(&rpacket->u.server_w3xp_clan_clanack.unknow1, 0);
@@ -225,7 +306,6 @@ extern int clan_send_status_window(t_connection * c)
 	    bn_byte_set(&rpacket->u.server_w3xp_clan_clanack.status, member->status);
 	    conn_push_outqueue(c, rpacket);
 	    packet_del_ref(rpacket);
-	}
     }
     return 0;
 }
@@ -233,20 +313,29 @@ extern int clan_send_status_window(t_connection * c)
 extern int clan_close_status_window(t_connection * c)
 {
     t_packet * rpacket;
-    char const *clienttag;
+    t_clienttag clienttag;
 
-    if ((clienttag = conn_get_clienttag(c)) && ((strcasecmp(clienttag, CLIENTTAG_WARCRAFT3) == 0) || (strcasecmp(clienttag, CLIENTTAG_WAR3XP) == 0)) && ((rpacket = packet_create(packet_class_bnet)) != NULL))
+    if (!(clienttag = conn_get_clienttag(c)))
     {
-	packet_set_size(rpacket, sizeof(t_server_w3xp_clan_clanleaveack));
-	packet_set_type(rpacket, SERVER_W3XP_CLAN_CLANLEAVEACK);
-	bn_byte_set(&rpacket->u.server_w3xp_clan_clanack.unknow1, SERVER_W3XP_CLAN_CLANLEAVEACK_UNKNOWN1);
+    	eventlog(eventlog_level_error,__FUNCTION__,"conn has NULL clienttag");
+	return -1;
+    }
+    
+    if ((clienttag != CLIENTTAG_WARCRAFT3_UINT) && (clienttag != CLIENTTAG_WAR3XP_UINT))
+    	return 0;
+	
+    if ((rpacket = packet_create(packet_class_bnet)))
+    {
+	packet_set_size(rpacket, sizeof(t_server_w3xp_clanquitnotify));
+	packet_set_type(rpacket, SERVER_W3XP_CLANQUITNOTIFY);
+	bn_byte_set(&rpacket->u.server_w3xp_clanquitnotify.status, SERVER_W3XP_CLANQUITNOTIFY_STATUS_REMOVED_FROM_CLAN);
 	conn_push_outqueue(c, rpacket);
 	packet_del_ref(rpacket);
     }
     return 0;
 }
 
-extern int clan_send_member_status(t_connection * c, t_packet const *const packet)
+extern int clan_send_memberlist(t_connection * c, t_packet const *const packet)
 {
     t_packet * rpacket;
     t_elem *curr;
@@ -258,25 +347,35 @@ extern int clan_send_member_status(t_connection * c, t_packet const *const packe
     char tmpstr[2];
     const char *append_str;
 
-    if ((account = conn_get_account(c)) == NULL)
+    if (!(account = conn_get_account(c)))
 	return -1;
 
-    if ((clan = account_get_clan(account)) == NULL)
+    if (!(clan = account_get_clan(account)))
 	return -1;
 
     if ((rpacket = packet_create(packet_class_bnet)))
     {
-	packet_set_size(rpacket, sizeof(t_server_w3xp_clan_memberreply));
-	packet_set_type(rpacket, SERVER_W3XP_CLAN_MEMBERREPLY);
-	bn_int_set(&rpacket->u.server_w3xp_clan_memberreply.count, bn_int_get(packet->u.client_w3xp_clan_memberreq.count));
+    	t_account * memberacc;
+	
+	packet_set_size(rpacket, sizeof(t_server_w3xp_clanmemberlist_reply));
+	packet_set_type(rpacket, SERVER_W3XP_CLANMEMBERLIST_REPLY);
+	bn_int_set(&rpacket->u.server_w3xp_clanmemberlist_reply.count, 
+	           bn_int_get(packet->u.client_w3xp_clanmemberlist_req.count));
 	LIST_TRAVERSE(clan->members, curr)
 	{
 	    if (!(member = elem_get_data(curr)))
 	    {
 		eventlog(eventlog_level_error, __FUNCTION__, "got NULL element in list");
-		return -1;
+		continue;
 	    }
-	    username = account_get_name(member->memberacc);
+
+	    if (!(memberacc = member->memberacc))
+	    {
+		eventlog(eventlog_level_error,__FUNCTION__,"member has NULL account");
+		continue;
+	    }
+
+	    username = account_get_name(memberacc);
 	    packet_append_string(rpacket, username);
 	    tmpstr[0] = member->status;
 	    append_str = clanmember_get_online_status(member, &tmpstr[1]);
@@ -287,7 +386,7 @@ extern int clan_send_member_status(t_connection * c, t_packet const *const packe
 		packet_append_string(rpacket, "");
 	    count++;
 	}
-	bn_byte_set(&rpacket->u.server_w3xp_clan_memberreply.member_count, count);
+	bn_byte_set(&rpacket->u.server_w3xp_clanmemberlist_reply.member_count, count);
 	conn_push_outqueue(c, rpacket);
 	packet_del_ref(rpacket);
 	return 0;
@@ -401,8 +500,15 @@ extern int clan_get_possible_member(t_connection * c, t_packet const *const pack
 	    if ((fr = elem_get_data(curr)) != NULL)
 	    {
 		t_account *fr_acc = friend_get_account(fr);
-		char const *clienttag;
-		if (fr->mutual && ((conn = connlist_find_connection_by_account(fr_acc)) != NULL) && (conn_get_channel(conn) == channel) && (!account_get_clan(fr_acc)) && (!account_get_creating_clan(fr_acc)) && (clienttag = conn_get_clienttag(conn)) && ((strcasecmp(clienttag, CLIENTTAG_WAR3XP) == 0) || (strcasecmp(clienttag, CLIENTTAG_WARCRAFT3) == 0)) && (username = account_get_name(fr_acc)))
+		t_clienttag clienttag;
+		if (fr->mutual 
+		    && ((conn = connlist_find_connection_by_account(fr_acc)) != NULL) 
+		    && (conn_get_channel(conn) == channel) 
+		    && (!account_get_clan(fr_acc)) 
+		    && (!account_get_creating_clan(fr_acc)) 
+		    && (clienttag = conn_get_clienttag(conn)) 
+		    && ((clienttag == CLIENTTAG_WAR3XP_UINT) || (clienttag ==  CLIENTTAG_WARCRAFT3_UINT)) 
+		    && (username = account_get_name(fr_acc)))
 		{
 		    friend_count++;
 		    packet_append_string(rpacket, username);
@@ -445,8 +551,8 @@ extern int clanmember_on_change_status(t_clanmember * member)
     {
 	char tmpstr[2];
 	const char *append_str;
-	packet_set_size(rpacket, sizeof(t_server_w3xp_clan_memberchangeack));
-	packet_set_type(rpacket, SERVER_W3XP_CLAN_MEMBERCHANGEACK);
+	packet_set_size(rpacket, sizeof(t_server_w3xp_clanmemberupdate));
+	packet_set_type(rpacket, SERVER_W3XP_CLANMEMBERUPDATE);
 	packet_append_string(rpacket, account_get_name(member->memberacc));
 	tmpstr[0] = member->status;
 	append_str = clanmember_get_online_status(member, &tmpstr[1]);
@@ -466,7 +572,7 @@ extern int clanmember_on_change_status_by_connection(t_connection * conn)
     t_packet * rpacket;
     t_account *acc;
     t_clanmember *member;
-    if (conn == NULL)
+    if (!(conn))
     {
 	eventlog(eventlog_level_error, __FUNCTION__, "got NULL conn");
 	return -1;
@@ -481,8 +587,8 @@ extern int clanmember_on_change_status_by_connection(t_connection * conn)
     {
 	char tmpstr[2];
 	const char *append_str;
-	packet_set_size(rpacket, sizeof(t_server_w3xp_clan_memberchangeack));
-	packet_set_type(rpacket, SERVER_W3XP_CLAN_MEMBERCHANGEACK);
+	packet_set_size(rpacket, sizeof(t_server_w3xp_clanmemberupdate));
+	packet_set_type(rpacket, SERVER_W3XP_CLANMEMBERUPDATE);
 	packet_append_string(rpacket, account_get_name(acc));
 	tmpstr[0] = member->status;
 	append_str = clanmember_get_online_status_by_connection(conn, &tmpstr[1]);
@@ -511,8 +617,8 @@ extern int clan_unload_members(t_clan * clan)
 		eventlog(eventlog_level_error, __FUNCTION__, "found NULL entry in list");
 		continue;
 	    }
-	    list_remove_elem(clan->members, curr);
-	    free((void *) member);
+	    list_remove_elem(clan->members, &curr);
+	    xfree((void *) member);
 	}
 
 	if (list_destroy(clan->members) < 0)
@@ -540,8 +646,8 @@ extern int clan_remove_all_members(t_clan * clan)
 	    }
 	    if (member->memberacc != NULL)
 		account_set_clanmember(member->memberacc, NULL);
-	    list_remove_elem(clan->members, curr);
-	    free((void *) member);
+	    list_remove_elem(clan->members, &curr);
+	    xfree((void *) member);
 	}
 
 	if (list_destroy(clan->members) < 0)
@@ -555,17 +661,17 @@ extern int clan_remove_all_members(t_clan * clan)
 
 extern int clanlist_remove_clan(t_clan * clan)
 {
+    t_elem * elem;
     if (clan == NULL)
     {
 	eventlog(eventlog_level_error, __FUNCTION__, "get NULL clan");
 	return -1;
     }
-    if (list_remove_data(clanlist_head, clan) < 0)
+    if (list_remove_data(clanlist_head, clan, &elem) < 0)
     {
 	eventlog(eventlog_level_error, __FUNCTION__, "could not delete clan entry");
 	return -1;
     }
-    list_purge(clanlist_head);
     return 0;
 }
 
@@ -578,7 +684,7 @@ extern int clan_save(t_clan * clan)
 {
     if (clan->created <= 0)
     {
-	if (time(NULL) - clan->creation_time > 120)
+	if (now - clan->creation_time > 120)
 	{
 	    clanlist_remove_clan(clan);
 	    clan_destroy(clan);
@@ -609,17 +715,7 @@ extern int clanlist_add_clan(t_clan * clan)
     if (!(clan->clanid))
 	clan->clanid = ++max_clanid;
 
-    if (list_append_data(clanlist_head, clan) < 0)
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not append item");
-	clan_remove_all_members(clan);
-	if (clan->clan_motd)
-	    free((void *) clan->clan_motd);
-	if (clan->clanname)
-	    free((void *) clan->clanname);
-	free((void *) clan);
-	return -1;
-    }
+    list_append_data(clanlist_head, clan);
 
     return clan->clanid;
 }
@@ -630,11 +726,7 @@ int clanlist_load(void)
     if (clanlist_head)
 	clanlist_unload();
 
-    if (!(clanlist_head = list_create()))
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not create clanlist");
-	return -1;
-    }
+    clanlist_head = list_create();
 
     storage->load_clans(_cb_load_clans);
 
@@ -679,12 +771,12 @@ extern int clanlist_unload(void)
 		continue;
 	    }
 	    if (clan->clanname)
-		free((void *) clan->clanname);
+		xfree((void *) clan->clanname);
 	    if (clan->clan_motd)
-		free((void *) clan->clan_motd);
+		xfree((void *) clan->clan_motd);
 	    clan_unload_members(clan);
-	    free((void *) clan);
-	    list_remove_elem(clanlist_head, curr);
+	    xfree((void *) clan);
+	    list_remove_elem(clanlist_head, &curr);
 	}
 
 	if (list_destroy(clanlist_head) < 0)
@@ -794,7 +886,7 @@ extern t_clanmember *clan_find_member_by_name(t_clan * clan, char const *membern
 	    eventlog(eventlog_level_error, __FUNCTION__, "got NULL element in list");
 	    return NULL;
 	}
-	if (strcmp(account_get_name(member->memberacc), membername) == 0)
+	if (strcasecmp(account_get_name(member->memberacc), membername) == 0)
 	    return member;
     }
 
@@ -852,28 +944,23 @@ extern int clanmember_set_account(t_clanmember * member, t_account * memberacc)
     return 0;
 }
 
-extern t_connection *clanmember_get_connection(t_clanmember * member)
+extern t_connection *clanmember_get_conn(t_clanmember * member)
 {
+    t_account * account;
+    
     if (!(member))
     {
 	eventlog(eventlog_level_error, __FUNCTION__, "got NULL clanmember");
 	return NULL;
     }
 
-    return (t_connection *) member->memberconn;
-}
-
-extern int clanmember_set_connection(t_clanmember * member, t_connection * memberconn)
-{
-    if (!(member))
+    if (!(account = member->memberacc))
     {
-	eventlog(eventlog_level_error, __FUNCTION__, "got NULL clanmember");
-	return -1;
+	eventlog(eventlog_level_error,__FUNCTION__,"member as NULL account");
+	return NULL;
     }
 
-    member->memberconn = memberconn;
-
-    return 0;
+    return account_get_conn(account);
 }
 
 extern char clanmember_get_status(t_clanmember * member)
@@ -884,7 +971,7 @@ extern char clanmember_get_status(t_clanmember * member)
 	return 0;
     }
 
-    if ((member->status == CLAN_NEW) && (time(NULL) - member->join_time > prefs_get_clan_newer_time() * 3600))
+    if ((member->status == CLAN_NEW) && (now - member->join_time > prefs_get_clan_newer_time() * 3600))
     {
 	member->status = CLAN_PEON;
 	member->clan->modified = 1;
@@ -939,12 +1026,12 @@ extern t_clan *clanmember_get_clan(t_clanmember * member)
 
 extern const char *clanmember_get_online_status(t_clanmember * member, char *status)
 {
-	return clanmember_get_online_status_by_connection(member->memberconn, status);
+	return clanmember_get_online_status_by_connection(clanmember_get_conn(member), status);
 }
 
 extern const char *clanmember_get_online_status_by_connection(t_connection * conn, char *status)
 {
-    if (conn)
+    if (conn && (conn_get_state(conn)!=conn_state_empty))
     {
 	t_game *game;
 	t_channel *channel;
@@ -979,9 +1066,8 @@ extern int clanmember_set_online(t_connection * c)
 	return -1;
     }
 
-    if ((acc = conn_get_account(c)) != NULL && ((member = account_get_clanmember(acc)) != NULL))
+    if ((acc = conn_get_account(c)) && (member = account_get_clanmember(acc)))
     {
-	clanmember_set_connection(member, c);
 	clanmember_on_change_status(member);
     }
 
@@ -999,9 +1085,8 @@ extern int clanmember_set_offline(t_connection * c)
 	return -1;
     }
 
-    if ((acc = conn_get_account(c)) != NULL && ((member = account_get_clanmember(acc)) != NULL))
+    if ((acc = conn_get_account(c)) && (member = account_get_clanmember_forced(acc)))
     {
-	clanmember_set_connection(member, NULL);
 	clanmember_on_change_status(member);
     }
 
@@ -1138,8 +1223,8 @@ extern int clan_set_motd(t_clan * clan, const char *motd)
     } else
     {
 	if (clan->clan_motd)
-	    free((void *) clan->clan_motd);
-	clan->clan_motd = strdup(motd);
+	    xfree((void *) clan->clan_motd);
+	clan->clan_motd = xstrdup(motd);
     }
     return 0;
 }
@@ -1149,7 +1234,7 @@ extern unsigned int clan_get_clanid(t_clan * clan)
     if (!(clan))
     {
 	eventlog(eventlog_level_error, __FUNCTION__, "got NULL clan");
-	return -1;
+	return 0;
     }
 
     return clan->clanid;
@@ -1179,7 +1264,7 @@ extern time_t clan_get_creation_time(t_clan * clan)
     return clan->creation_time;
 }
 
-extern t_clanmember *clan_add_member(t_clan * clan, t_account * memberacc, t_connection * memberconn, char status)
+extern t_clanmember *clan_add_member(t_clan * clan, t_account * memberacc, char status)
 {
     t_clanmember *member;
 
@@ -1195,27 +1280,16 @@ extern t_clanmember *clan_add_member(t_clan * clan, t_account * memberacc, t_con
 	return NULL;
     }
 
-    if (!(member = malloc(sizeof(t_clanmember))))
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for clanmember");
-	return NULL;
-    }
-
+    member = xmalloc(sizeof(t_clanmember));
     member->memberacc = memberacc;
-    member->memberconn = memberconn;
     member->status = status;
-    member->join_time = time(0);
+    member->join_time = now;
     member->clan = clan;
 #ifdef WITH_SQL
     member->modified = 1;
 #endif
 
-    if (list_append_data(clan->members, member) < 0)
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not append item");
-	free((void *) member);
-	return NULL;
-    }
+    list_append_data(clan->members, member);
 
     account_set_clanmember(memberacc, member);
 
@@ -1226,9 +1300,11 @@ extern t_clanmember *clan_add_member(t_clan * clan, t_account * memberacc, t_con
 
 extern int clan_remove_member(t_clan * clan, t_clanmember * member)
 {
+    t_elem * elem;
+    
     if (!member)
 	return -1;
-    if (list_remove_data(clan->members, member) < 0)
+    if (list_remove_data(clan->members, member, &elem) < 0)
     {
 	eventlog(eventlog_level_error, __FUNCTION__, "could not remove member");
 	return -1;
@@ -1238,84 +1314,44 @@ extern int clan_remove_member(t_clan * clan, t_clanmember * member)
 	account_set_clanmember(member->memberacc, NULL);
 	storage->remove_clanmember(account_get_uid(member->memberacc));
     }
-    free((void *) member);
-    list_purge(clan->members);
+    xfree((void *) member);
     clan->modified = 1;
     return 0;
 }
 
-extern t_clan *clan_create(t_account * chieftain_acc, t_connection * chieftain_conn, int clantag, const char *clanname, const char *motd)
+extern t_clan *clan_create(t_account * chieftain_acc, int clantag, const char *clanname, const char *motd)
 {
     t_clan *clan;
     t_clanmember *member;
 
-    if (!(clan = malloc(sizeof(t_clan))))
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for new clan");
-	return NULL;
-    }
-
-    if (!(member = malloc(sizeof(t_clanmember))))
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for clanmember");
-	free((void *) clan);
-	return NULL;
-    }
+    clan = xmalloc(sizeof(t_clan));
+    member = xmalloc(sizeof(t_clanmember));
 
     if (!(clanname))
     {
 	eventlog(eventlog_level_error, __FUNCTION__, "got NULL clanname");
-	free((void *) clan);
-	free((void *) member);
+	xfree((void *) clan);
+	xfree((void *) member);
 	return NULL;
     }
 
-    if (!(clan->clanname = strdup(clanname)))
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for clanname");
-	free((void *) clan);
-	free((void *) member);
-	return NULL;
-    }
+    clan->clanname = xstrdup(clanname);
 
     if (!(motd))
-    {
-	clan->clan_motd = strdup("This is a newly created clan");
-    } else
-    {
-	clan->clan_motd = strdup(motd);
-    }
-    if (!(clan->clan_motd))
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for clan_motd");
-	if (clan->clanname)
-	    free((void *) clan->clanname);
-	free((void *) member);
-	free((void *) clan);
-	return NULL;
-    }
+	clan->clan_motd = xstrdup("This is a newly created clan");
+    else
+	clan->clan_motd = xstrdup(motd);
 
-    clan->creation_time = time(0);
+    clan->creation_time = now;
     clan->clantag = clantag;
     clan->clanid = ++max_clanid;
     clan->created = 0;
     clan->modified = 1;
     clan->channel_type = prefs_get_clan_channel_default_private();
 
-    if ((clan->members = list_create()) == NULL)
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not create memberlist");
-	if (clan->clanname)
-	    free((void *) clan->clanname);
-	if (clan->clan_motd)
-	    free((void *) clan->clan_motd);
-	free((void *) member);
-	free((void *) clan);
-	return NULL;
-    }
+    clan->members = list_create();
 
     member->memberacc = chieftain_acc;
-    member->memberconn = chieftain_conn;
     member->status = CLAN_CHIEFTAIN;
     member->join_time = clan->creation_time;
     member->clan = clan;
@@ -1323,26 +1359,8 @@ extern t_clan *clan_create(t_account * chieftain_acc, t_connection * chieftain_c
     member->modified = 1;
 #endif
 
-    if (list_append_data(clan->members, member) < 0)
-    {
-	t_elem *curr;
-	eventlog(eventlog_level_error, __FUNCTION__, "could not append item");
-	if (clan->clanname)
-	    free((void *) clan->clanname);
-	if (clan->clan_motd)
-	    free((void *) clan->clan_motd);
-	free((void *) member);
-	LIST_TRAVERSE(clan->members, curr)
-	{
-	    t_clanmember *member;
-	    if ((member = elem_get_data(curr)) != NULL)
-		free(member);
-	    list_remove_elem(clan->members, curr);
-	}
-	list_destroy(clan->members);
-	free((void *) clan);
-	return NULL;
-    }
+    list_append_data(clan->members, member);
+
     account_set_clanmember(chieftain_acc, member);
 
     return clan;
@@ -1353,11 +1371,11 @@ extern int clan_destroy(t_clan * clan)
     if (!clan)
 	return 0;
     if (clan->clanname)
-	free((void *) clan->clanname);
+	xfree((void *) clan->clanname);
     if (clan->clan_motd)
-	free((void *) clan->clan_motd);
+	xfree((void *) clan->clan_motd);
     clan_remove_all_members(clan);
-    free((void *) clan);
+    xfree((void *) clan);
     return 0;
 }
 
