@@ -47,19 +47,22 @@
 #include "common/bnet_protocol.h"
 #include "common/util.h"
 #include "common/bn_type.h"
+#include "common/xalloc.h"
 #include "game_conv.h"
 #include "common/setup_after.h"
 
 
-extern t_game_type bngreqtype_to_gtype(char const * clienttag, unsigned short bngtype)
+extern t_game_type bngreqtype_to_gtype(t_clienttag clienttag, unsigned short bngtype)
 {
+    char clienttag_str[5];
+
     if (!clienttag)
     {
-	eventlog(eventlog_level_error,"bngreqtype_to_gtype","got NULL clienttag");
+	eventlog(eventlog_level_error,__FUNCTION__,"got NULL clienttag");
 	return game_type_none;
     }
     
-    if (strcmp(clienttag,CLIENTTAG_WARCIIBNE)==0)
+    if (clienttag==CLIENTTAG_WARCIIBNE_UINT)
     {
 	switch (bngtype)
 	{
@@ -80,41 +83,50 @@ extern t_game_type bngreqtype_to_gtype(char const * clienttag, unsigned short bn
 	case CLIENT_GAMELISTREQ_MAPSET:
 	    return game_type_mapset;
 	default:
-	    eventlog(eventlog_level_error,"bngreqtype_to_gtype","unknown Warcraft bnet game type \"%s\" %hu",clienttag,bngtype);
+	    eventlog(eventlog_level_error,__FUNCTION__,"unknown Warcraft bnet game type \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),bngtype);
 	    return game_type_none;
 	}
     }
-    else if (strcmp(clienttag,CLIENTTAG_DIABLO2DV)==0 ||
-	     strcmp(clienttag,CLIENTTAG_DIABLO2XP)==0)
+    else if (clienttag==CLIENTTAG_DIABLO2DV_UINT ||
+	     clienttag==CLIENTTAG_DIABLO2XP_UINT)
     {
 	switch (bngtype)
 	{
 	case CLIENT_GAMELISTREQ_ALL:
 	    return game_type_diablo2open;
 	default:
-	    eventlog(eventlog_level_error,"bngreqtype_to_gtype","unknown Diablo II bnet game type \"%s\" %hu",clienttag,bngtype);
+	    eventlog(eventlog_level_error,__FUNCTION__,"unknown Diablo II bnet game type \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),bngtype);
 	    return game_type_none;
 	}
     }
-    else if (strcmp(clienttag,CLIENTTAG_DIABLORTL)==0 ||
-	     strcmp(clienttag,CLIENTTAG_DIABLOSHR)==0)
+    else if (clienttag==CLIENTTAG_DIABLORTL_UINT ||
+	     clienttag==CLIENTTAG_DIABLOSHR_UINT)
     {
 	switch (bngtype)
 	{
-	case CLIENT_GAMELISTREQ_ALL:
-	    return game_type_all;
+	case CLIENT_GAMETYPE_DIABLO_0:
 	case CLIENT_GAMETYPE_DIABLO_1:
 	case CLIENT_GAMETYPE_DIABLO_2:
 	case CLIENT_GAMETYPE_DIABLO_3:
+	case CLIENT_GAMETYPE_DIABLO_4:
+	case CLIENT_GAMETYPE_DIABLO_5:
+	case CLIENT_GAMETYPE_DIABLO_6:
+	case CLIENT_GAMETYPE_DIABLO_7:
+	case CLIENT_GAMETYPE_DIABLO_8:
+	case CLIENT_GAMETYPE_DIABLO_9:
+	case CLIENT_GAMETYPE_DIABLO_a:
+	case CLIENT_GAMETYPE_DIABLO_b:
+	case CLIENT_GAMETYPE_DIABLO_c:
+	case CLIENT_GAMETYPE_DIABLO_d:
 	    return game_type_diablo;
 	default:
-	    eventlog(eventlog_level_error,"bngreqtype_to_gtype","unknown Diablo bnet game type \"%s\" %hu",clienttag,bngtype);
+	    eventlog(eventlog_level_error,__FUNCTION__,"unknown Diablo bnet game type \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),bngtype);
 	    return game_type_none;
 	}
     }
-    else if (strcmp(clienttag,CLIENTTAG_STARCRAFT)==0 ||
-	     strcmp(clienttag,CLIENTTAG_BROODWARS)==0 ||
-	     strcmp(clienttag,CLIENTTAG_SHAREWARE)==0)
+    else if (clienttag==CLIENTTAG_STARCRAFT_UINT ||
+	     clienttag==CLIENTTAG_BROODWARS_UINT ||
+	     clienttag==CLIENTTAG_SHAREWARE_UINT)
     {
 	switch (bngtype)
 	{
@@ -149,34 +161,33 @@ extern t_game_type bngreqtype_to_gtype(char const * clienttag, unsigned short bn
 	case CLIENT_GAMELISTREQ_TOPVBOT:
 	    return game_type_topvbot;
 	default:
-	    eventlog(eventlog_level_error,"bngreqtype_to_gtype","unknown Starcraft bnet game type \"%s\" %hu",clienttag,bngtype);
+	    eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game type \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),bngtype);
 	    return game_type_none;
 	}
     }
-
-    /* ADDED BY UNDYING SOULZZ 4/3/02 */
-    else if ( strcmp( clienttag, CLIENTTAG_WARCRAFT3 ) == 0 || strcmp( clienttag, CLIENTTAG_WAR3XP ) == 0 )
+    else if (clienttag==CLIENTTAG_WARCRAFT3_UINT ||
+	     clienttag==CLIENTTAG_WAR3XP_UINT)
     {
 	return game_type_all;
     }
-
     else
     {
-	eventlog(eventlog_level_error,"bngreqtype_to_gtype","unknown game clienttag \"%s\" %hu",clienttag,bngtype);
+	eventlog(eventlog_level_error,__FUNCTION__,"unknown game clienttag \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),bngtype);
 	return game_type_none;
     }
 }
 
 
-extern t_game_type bngtype_to_gtype(char const * clienttag, unsigned short bngtype)
+extern t_game_type bngtype_to_gtype(t_clienttag clienttag, unsigned short bngtype)
 {
+    char clienttag_str[5];
     if (!clienttag)
     {
-	eventlog(eventlog_level_error,"bngtype_to_gtype","got NULL clienttag");
+	eventlog(eventlog_level_error,__FUNCTION__,"got NULL clienttag");
 	return game_type_none;
     }
     
-    if (strcmp(clienttag,CLIENTTAG_WARCIIBNE)==0)
+    if (clienttag==CLIENTTAG_WARCIIBNE_UINT)
     {
 	switch (bngtype)
 	{
@@ -195,12 +206,12 @@ extern t_game_type bngtype_to_gtype(char const * clienttag, unsigned short bngty
 	case CLIENT_GAMELISTREQ_MAPSET:
 	    return game_type_mapset;
 	default:
-	    eventlog(eventlog_level_error,"bngtype_to_gtype","unknown Warcraft bnet game type \"%s\" %hu",clienttag,bngtype);
+	    eventlog(eventlog_level_error,__FUNCTION__,"unknown Warcraft bnet game type \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),bngtype);
 	    return game_type_none;
 	}
     }
-    else if (strcmp(clienttag,CLIENTTAG_DIABLO2DV)==0 ||
-	     strcmp(clienttag,CLIENTTAG_DIABLO2XP)==0)
+    else if (clienttag==CLIENTTAG_DIABLO2DV_UINT ||
+	     clienttag==CLIENTTAG_DIABLO2XP_UINT)
     {
 	switch (bngtype)
 	{
@@ -211,27 +222,37 @@ extern t_game_type bngtype_to_gtype(char const * clienttag, unsigned short bngty
         case CLIENT_GAMETYPE_DIABLO2_CLOSE:
             return game_type_diablo2closed;
 	default:
-	    eventlog(eventlog_level_error,"bngtype_to_gtype","unknown Diablo II bnet game type \"%s\" %hu",clienttag,bngtype);
+	    eventlog(eventlog_level_error,__FUNCTION__,"unknown Diablo II bnet game type \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),bngtype);
 	    return game_type_none;
 	}
     }
-    else if (strcmp(clienttag,CLIENTTAG_DIABLORTL)==0 ||
-	     strcmp(clienttag,CLIENTTAG_DIABLOSHR)==0)
+    else if (clienttag==CLIENTTAG_DIABLORTL_UINT ||
+	     clienttag==CLIENTTAG_DIABLOSHR_UINT)
     {
 	switch (bngtype)
 	{
+	case CLIENT_GAMETYPE_DIABLO_0:
 	case CLIENT_GAMETYPE_DIABLO_1:
 	case CLIENT_GAMETYPE_DIABLO_2:
 	case CLIENT_GAMETYPE_DIABLO_3:
+	case CLIENT_GAMETYPE_DIABLO_4:
+	case CLIENT_GAMETYPE_DIABLO_5:
+	case CLIENT_GAMETYPE_DIABLO_6:
+	case CLIENT_GAMETYPE_DIABLO_7:
+	case CLIENT_GAMETYPE_DIABLO_8:
+	case CLIENT_GAMETYPE_DIABLO_9:
+	case CLIENT_GAMETYPE_DIABLO_a:
+	case CLIENT_GAMETYPE_DIABLO_b:
+	case CLIENT_GAMETYPE_DIABLO_c:
 	    return game_type_diablo;
 	default:
-	    eventlog(eventlog_level_error,"bngtype_to_gtype","unknown Diablo bnet game type \"%s\" %hu",clienttag,bngtype);
+	    eventlog(eventlog_level_error,__FUNCTION__,"unknown Diablo bnet game type \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),bngtype);
 	    return game_type_none;
 	}
     }
-    else if (strcmp(clienttag,CLIENTTAG_STARCRAFT)==0 ||
-	     strcmp(clienttag,CLIENTTAG_BROODWARS)==0 ||
-	     strcmp(clienttag,CLIENTTAG_SHAREWARE)==0)
+    else if (clienttag==CLIENTTAG_STARCRAFT_UINT ||
+	     clienttag==CLIENTTAG_BROODWARS_UINT ||
+	     clienttag==CLIENTTAG_SHAREWARE_UINT)
     {
 	switch (bngtype)
 	{
@@ -266,20 +287,18 @@ extern t_game_type bngtype_to_gtype(char const * clienttag, unsigned short bngty
 	case CLIENT_GAMELISTREQ_TOPVBOT:
 	    return game_type_topvbot;
 	default:
-	    eventlog(eventlog_level_error,"bngtype_to_gtype","unknown Starcraft bnet game type \"%s\" %hu",clienttag,bngtype);
+	    eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game type \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),bngtype);
 	    return game_type_none;
 	}
     }
-
-    /* ADDED BY UNDYING SOULZZ 4/3/02 */
-    else if ( strcmp( clienttag, CLIENTTAG_WARCRAFT3 ) == 0 || strcmp( clienttag, CLIENTTAG_WAR3XP ) == 0)
+    else if (clienttag==CLIENTTAG_WARCRAFT3_UINT ||
+	     clienttag==CLIENTTAG_WAR3XP_UINT)
     {
 	return game_type_all;
     }
-
     else
     {
-	eventlog(eventlog_level_error,"bngtype_to_gtype","unknown game clienttag \"%s\" %hu",clienttag,bngtype);
+	eventlog(eventlog_level_error,__FUNCTION__,"unknown game clienttag \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),bngtype);
 	return game_type_none;
     }
 }
@@ -322,57 +341,80 @@ extern unsigned short gtype_to_bngtype(t_game_type gtype)
     case game_type_diablo:
 	return CLIENT_GAMELISTREQ_DIABLO;
     case game_type_diablo2open:
+    	return SERVER_GAMELISTREPLY_TYPE_DIABLO2_OPEN;
     case game_type_diablo2closed:
-        eventlog(eventlog_level_error,"gtype_to_bngtype","don't know how to list Diablo II");
+        eventlog(eventlog_level_error,__FUNCTION__,"don't know how to list Diablo II");
 	return 0;
     case game_type_anongame:
 	return 0;
     case game_type_none:
     default:
-        eventlog(eventlog_level_error,"gtype_to_bngtype","unknown game type %u",(unsigned int)gtype);
+        eventlog(eventlog_level_error,__FUNCTION__,"unknown game type %u",(unsigned int)gtype);
 	return 0xffff;
     }
 }
 
 
-extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gtype, unsigned short bngoption)
+extern t_game_option bngoption_to_goption(t_clienttag clienttag, t_game_type gtype, unsigned short bngoption)
 {
+    char clienttag_str[5];
+    
     if (!clienttag)
     {
-	eventlog(eventlog_level_error,"bngoption_to_goption","got NULL clienttag");
+	eventlog(eventlog_level_error,__FUNCTION__,"got NULL clienttag");
 	return game_option_none;
     }
     
-    if (strcmp(clienttag,CLIENTTAG_WARCIIBNE)==0)
+    if (clienttag==CLIENTTAG_WARCIIBNE_UINT)
     {
 	switch (gtype)
 	{
 	case game_type_topvbot:
 	    switch (bngoption)
 	    {
-	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Warcraft II bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+	      case CLIENT_STARTGAME4_OPTION_TOPVBOT_1:
+	        return game_option_topvbot_1;
+	      case CLIENT_STARTGAME4_OPTION_TOPVBOT_2:
+		return game_option_topvbot_2;
+	      case CLIENT_STARTGAME4_OPTION_TOPVBOT_3:
+	        return game_option_topvbot_3;
+	      case CLIENT_STARTGAME4_OPTION_TOPVBOT_4:
+		return game_option_topvbot_4;
+	      case CLIENT_STARTGAME4_OPTION_TOPVBOT_5:
+	        return game_option_topvbot_5;
+	      case CLIENT_STARTGAME4_OPTION_TOPVBOT_6:
+		return game_option_topvbot_6;
+	      case CLIENT_STARTGAME4_OPTION_TOPVBOT_7:
+	        return game_option_topvbot_7;
+	      default:
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Warcraft II bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_melee:
 	    switch (bngoption)
 	    {
+	    case CLIENT_STARTGAME4_OPTION_MELEE_NORMAL:
+		return game_option_melee_normal;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Warcraft II bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Warcraft II bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_ffa:
 	    switch (bngoption)
 	    {
+	    case CLIENT_STARTGAME4_OPTION_FFA_NORMAL:
+		return game_option_ffa_normal;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Warcraft II bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Warcraft II bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_oneonone:
 	    switch (bngoption)
 	    {
+	    case CLIENT_STARTGAME4_OPTION_ONEONONE_NORMAL:
+		return game_option_oneonone_normal;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Warcraft II bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Warcraft II bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_ladder:
@@ -383,30 +425,32 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	    case CLIENT_STARTGAME4_OPTION_LADDER_NOPENALTY:
 		return game_option_ladder_nopenalty;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Warcraft II bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Warcraft II bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_ironman:
 	    switch (bngoption)
 	    {
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Warcraft II bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Warcraft II bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_mapset:
 	    switch (bngoption)
 	    {
+	    case CLIENT_STARTGAME4_OPTION_MAPSET_NORMAL:
+		return game_option_mapset_normal;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Warcraft II bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Warcraft II bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	default:
-	    eventlog(eventlog_level_error,"bngoption_to_goption","unknown Warcraft II game type \"%s\" %u",clienttag,(unsigned int)gtype);
+	    eventlog(eventlog_level_error,__FUNCTION__,"unknown Warcraft II game type \"%s\" %u",tag_uint_to_str(clienttag_str,clienttag),(unsigned int)gtype);
 	    return game_option_none;
 	}
     }
-    else if (strcmp(clienttag,CLIENTTAG_DIABLO2DV)==0 ||
-	     strcmp(clienttag,CLIENTTAG_DIABLO2XP)==0)
+    else if (clienttag==CLIENTTAG_DIABLO2DV_UINT ||
+	     clienttag==CLIENTTAG_DIABLO2XP_UINT)
     {
 	switch (gtype)
 	{
@@ -416,7 +460,7 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
             case CLIENT_STARTGAME4_OPTION_NONE: /* FIXME: really? */
                 return game_option_none;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Diablo II bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Diablo II bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_diablo2closed:
@@ -425,16 +469,16 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
             case CLIENT_STARTGAME4_OPTION_NONE: /* FIXME: really? */
                 return game_option_none;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Diablo II bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Diablo II bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	default:
-	    eventlog(eventlog_level_error,"bngoption_to_goption","unknown Diablo II game type \"%s\" %hu",clienttag,(unsigned int)gtype);
+	    eventlog(eventlog_level_error,__FUNCTION__,"unknown Diablo II game type \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),(unsigned int)gtype);
 	    return game_option_none;
 	}
     }
-    else if (strcmp(clienttag,CLIENTTAG_DIABLORTL)==0 ||
-	     strcmp(clienttag,CLIENTTAG_DIABLOSHR)==0)
+    else if (clienttag==CLIENTTAG_DIABLORTL_UINT ||
+	     clienttag==CLIENTTAG_DIABLOSHR_UINT)
     {
 	switch (gtype)
 	{
@@ -442,17 +486,17 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	    switch (bngoption)
 	    {
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Diablo bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		/* diablo doesn't use any options */
 		return game_option_none;
 	    }
 	default:
-	    eventlog(eventlog_level_error,"bngoption_to_goption","unknown Diablo game type \"%s\" %hu",clienttag,(unsigned int)gtype);
+	    eventlog(eventlog_level_error,__FUNCTION__,"unknown Diablo game type \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),(unsigned int)gtype);
 	    return game_option_none;
 	}
     }
-    else if (strcmp(clienttag,CLIENTTAG_STARCRAFT)==0 ||
-	     strcmp(clienttag,CLIENTTAG_BROODWARS)==0 ||
-	     strcmp(clienttag,CLIENTTAG_SHAREWARE)==0)
+    else if (clienttag==CLIENTTAG_STARCRAFT_UINT ||
+	     clienttag==CLIENTTAG_BROODWARS_UINT ||
+	     clienttag==CLIENTTAG_SHAREWARE_UINT)
     {
 	switch (gtype)
 	{
@@ -462,7 +506,7 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	    case CLIENT_STARTGAME4_OPTION_MELEE_NORMAL:
 		return game_option_melee_normal;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_ffa:
@@ -471,7 +515,7 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	    case CLIENT_STARTGAME4_OPTION_FFA_NORMAL:
 		return game_option_ffa_normal;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_oneonone:
@@ -480,7 +524,7 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	    case CLIENT_STARTGAME4_OPTION_ONEONONE_NORMAL:
 		return game_option_oneonone_normal;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_ctf:
@@ -489,7 +533,7 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	    case CLIENT_STARTGAME4_OPTION_CTF_NORMAL:
 		return game_option_ctf_normal;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_greed:
@@ -504,7 +548,7 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	    case CLIENT_STARTGAME4_OPTION_GREED_2500:
 		return game_option_greed_2500;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_slaughter:
@@ -519,7 +563,7 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	    case CLIENT_STARTGAME4_OPTION_SLAUGHTER_15:
 		return game_option_slaughter_15;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_sdeath:
@@ -528,7 +572,7 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	    case CLIENT_STARTGAME4_OPTION_SDEATH_NORMAL:
 		return game_option_sdeath_normal;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_ladder:
@@ -539,7 +583,7 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	    case CLIENT_STARTGAME4_OPTION_LADDER_NOPENALTY:
 		return game_option_ladder_nopenalty;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_mapset:
@@ -548,7 +592,7 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	    case CLIENT_STARTGAME4_OPTION_MAPSET_NORMAL:
 		return game_option_mapset_normal;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_teammelee:
@@ -561,7 +605,7 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	    case CLIENT_STARTGAME4_OPTION_TEAMMELEE_2:
 		return game_option_teammelee_2;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_teamffa:
@@ -574,7 +618,7 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	    case CLIENT_STARTGAME4_OPTION_TEAMFFA_2:
 		return game_option_teamffa_2;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_teamctf:
@@ -587,14 +631,14 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	    case CLIENT_STARTGAME4_OPTION_TEAMCTF_2:
 		return game_option_teamctf_2;
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_pgl:
 	    switch (bngoption)
 	    {
 	    default:
-		eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+		eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_topvbot:
@@ -615,25 +659,23 @@ extern t_game_option bngoption_to_goption(char const * clienttag, t_game_type gt
 	      case CLIENT_STARTGAME4_OPTION_TOPVBOT_7:
 	        return game_option_topvbot_7;
             default:
-	     eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",clienttag,game_type_get_str(gtype),bngoption);
+	     eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft bnet game option for \"%s\" game \"%s\" %hu",tag_uint_to_str(clienttag_str,clienttag),game_type_get_str(gtype),bngoption);
 		return game_option_none;
 	    }
 	case game_type_none:
 	default:
-	    eventlog(eventlog_level_error,"bngoption_to_goption","unknown Starcraft game type \"%s\" %u(%s)",clienttag,(unsigned int)gtype,game_type_get_str(gtype));
+	    eventlog(eventlog_level_error,__FUNCTION__,"unknown Starcraft game type \"%s\" %u(%s)",tag_uint_to_str(clienttag_str,clienttag),(unsigned int)gtype,game_type_get_str(gtype));
 	    return game_option_none;
 	}
     }
-   
-    /* ADDED BY UNDYING SOULZZ 4/3/02 */
-    else if ( strcmp( clienttag, CLIENTTAG_WARCRAFT3 ) == 0 || strcmp( clienttag, CLIENTTAG_WAR3XP ) == 0 )
+    else if (clienttag==CLIENTTAG_WARCRAFT3_UINT ||
+	     clienttag==CLIENTTAG_WAR3XP_UINT)
     {
  	return game_option_none;
     }
-
     else
     {
-	eventlog(eventlog_level_error,"bngoption_to_goption","unknown game clienttag \"%s\" %u",clienttag,(unsigned int)gtype);
+	eventlog(eventlog_level_error,__FUNCTION__,"unknown game tag_uint_to_str(clienttag_str,clienttag) \"%s\" %u",clienttag,(unsigned int)gtype);
 	return game_option_none;
     }
 }
@@ -656,7 +698,7 @@ extern t_game_result bngresult_to_gresult(unsigned int bngresult)
     case CLIENT_GAME_REPORT_RESULT_OBSERVER:
 	return game_result_observer;
     default:
-	eventlog(eventlog_level_error,"bngresult_to_gresult","unknown bnet game result %u",bngresult);
+	eventlog(eventlog_level_error,__FUNCTION__,"unknown bnet game result %u",bngresult);
 	return game_result_disconnect; /* bad packet? */
     }
 }
@@ -674,6 +716,10 @@ extern t_game_maptype bngmaptype_to_gmaptype(unsigned int bngmaptype)
         return game_maptype_ladder;
     case CLIENT_MAPTYPE_PGL:
         return game_maptype_pgl;
+    case CLIENT_MAPTYPE_KBK:
+	return game_maptype_kbk;
+    case CLIENT_MAPTYPE_CompUSA:
+	return game_maptype_compusa;
     default:
         return game_maptype_none;
     }
@@ -770,7 +816,7 @@ static const char * _w3_decrypt_mapinfo(const char *enc)
     unsigned    pos;
     unsigned char bitmask;
 
-    if (!(mapinfo = strdup(enc))) {
+    if (!(mapinfo = xstrdup(enc))) {
 	eventlog(eventlog_level_error, __FUNCTION__, "not enough memory to setup temporary buffer");
 	return NULL;
     }
@@ -799,7 +845,7 @@ static const char * _w3_decrypt_mapinfo(const char *enc)
 
 extern int game_parse_info(t_game * game, char const * gameinfo)
 {
-    char const * clienttag;
+    t_clienttag  clienttag;
     char *       save;
     char *       line1;
     char *       line2;
@@ -823,21 +869,15 @@ extern int game_parse_info(t_game * game, char const * gameinfo)
     
     if (!game)
     {
-	eventlog(eventlog_level_error,"game_parse_info","got NULL game");
+	eventlog(eventlog_level_error,__FUNCTION__,"got NULL game");
 	return -1;
     }
     if (!gameinfo)
     {
-	eventlog(eventlog_level_error,"game_parse_info","got NULL gameinfo");
+	eventlog(eventlog_level_error,__FUNCTION__,"got NULL gameinfo");
 	return -1;
     }
     
-	// [quetzal] 20020831
-	if (!strlen(gameinfo)) 
-	{
-		eventlog(eventlog_level_info,"game_parse_info", "got empty gameinfo (W3 client joining CG)");
-		return -1;
-	}
 /*
 
 BW 104:
@@ -914,21 +954,23 @@ Also, what is the upper player limit on WCII... 8 like on Starcraft?
     
     if (!(clienttag = game_get_clienttag(game)))
     {
-	eventlog(eventlog_level_error,"game_parse_info","NULL clienttag for game?");
+	eventlog(eventlog_level_error,__FUNCTION__,"NULL clienttag for game?");
 	return -1;
     }
 
-    if (strcmp(clienttag,CLIENTTAG_DIABLORTL)==0 ||
-	strcmp(clienttag,CLIENTTAG_DIABLOSHR)==0)
+    if (clienttag==CLIENTTAG_DIABLORTL_UINT ||
+	clienttag==CLIENTTAG_DIABLOSHR_UINT)
     {
 	game_set_maxplayers(game,4);
-	eventlog(eventlog_level_debug,"game_parse_info","no gameinfo for Diablo");
+	eventlog(eventlog_level_debug,__FUNCTION__,"no gameinfo for Diablo");
 	return 0;
     }
-    else if (strcmp(clienttag,CLIENTTAG_DIABLO2DV)==0 ||
-	     strcmp(clienttag,CLIENTTAG_DIABLO2XP)==0)
+    else if (clienttag==CLIENTTAG_DIABLO2DV_UINT ||
+	     clienttag==CLIENTTAG_DIABLO2XP_UINT)
     {
-        if (game->type == game_type_diablo2closed)
+        if ((game->type == game_type_diablo2closed) &&
+	    (!strlen(gameinfo)))
+	
 	{
 	  /* D2 closed games are handled by d2cs so we can have only have a generic startgame4
 	     without any info :(, this fix also a memory leak for description allocation */
@@ -939,21 +981,33 @@ Also, what is the upper player limit on WCII... 8 like on Starcraft?
 	{
 	  char         difficulty[2];
 	  unsigned int bngdifficulty;
+	  
+	  if (!strlen(gameinfo)) 
+	  {
+	      eventlog(eventlog_level_info,__FUNCTION__, "got empty gameinfo (from D2 client)");
+	      return -1;
+	  }
 	
 	  difficulty[0] = gameinfo[0];
 	  difficulty[1] = '\0';
 	  if (str_to_uint(difficulty,&bngdifficulty)<0)
 	    {
-	      eventlog(eventlog_level_error,"game_parse_info","bad gameinfo format (missing difficulty) \"%s\"",gameinfo);
+	      eventlog(eventlog_level_error,__FUNCTION__,"bad gameinfo format (missing difficulty) \"%s\"",gameinfo);
 	      return -1;
 	    }
 	  game_set_difficulty(game,bngdifficulty);
 	  game_set_description(game,&gameinfo[1]);
+
+	  if ((game->type == game_type_diablo2closed))
+	  {
+	    eventlog(eventlog_level_debug,__FUNCTION__,"D2 bug workarround needed (open games tagged as closed)");
+	    game->type = game_type_diablo2open;
+	  }
 	}
 	return 0;
     }
-
-    if (strcmp(clienttag,CLIENTTAG_WARCRAFT3)==0 || strcmp(clienttag,CLIENTTAG_WAR3XP)==0)
+    else if (clienttag==CLIENTTAG_WARCRAFT3_UINT ||
+	     clienttag==CLIENTTAG_WAR3XP_UINT)
     {
 /*  Warcraft 3 game info format -- by Soar
     0x00 -- 1 byte  (char, Empty slots)
@@ -977,6 +1031,12 @@ If the corresponding bit is a '0' then subtract 1 from the character.
 */
         const char *pstr;
 
+	if (!strlen(gameinfo)) 
+	{
+	    eventlog(eventlog_level_info,__FUNCTION__, "got empty gameinfo (from W3 client)");
+	    return -1;
+	}
+    
 	if (strlen(gameinfo) < 0xf + 2 + 1 + 2 + 4) {
 	    eventlog(eventlog_level_error, __FUNCTION__, "got too short W3 mapinfo");
 	    return -1;
@@ -990,28 +1050,28 @@ If the corresponding bit is a '0' then subtract 1 from the character.
         game_set_mapsize_x(game, bn_short_get(*((bn_short*)(pstr + 5))));
         game_set_mapsize_y(game, bn_short_get(*((bn_short*)(pstr + 7))));
         game_set_mapname(game, pstr + 13);
-	free((void*)pstr);
+	xfree((void*)pstr);
 
 	return 0;
     }
-    
+
     /* otherwise it's Starcraft, Brood War, or Warcraft II */
-    if (!(save = strdup(gameinfo)))
+    if (!(save = xstrdup(gameinfo)))
     {
-	eventlog(eventlog_level_error,"game_parse_info","could not allocate memory for save");
+	eventlog(eventlog_level_error,__FUNCTION__,"could not allocate memory for save");
 	return -1;
     }
     
     if (!(line1  = strtok(save,"\r"))) /* actual game info fields */
     {
-	eventlog(eventlog_level_error,"game_parse_info","bad gameinfo format (missing line1) \"%s\"",gameinfo);
-	free(save);
+	eventlog(eventlog_level_error,__FUNCTION__,"bad gameinfo format (missing line1) \"%s\"",gameinfo);
+	xfree(save);
 	return -1;
     }
     if (!(line2  = strtok(NULL,"\r")))
     {
-	eventlog(eventlog_level_error,"game_parse_info","bad gameinfo format (missing player) \"%s\"",gameinfo);
-	free(save);
+	eventlog(eventlog_level_error,__FUNCTION__,"bad gameinfo format (missing player) \"%s\"",gameinfo);
+	xfree(save);
 	return -1;
     }
     /* is there room for another field after that? */
@@ -1026,112 +1086,102 @@ If the corresponding bit is a '0' then subtract 1 from the character.
     currtok = line1;
     if (!(unknown    = strsep(&currtok,","))) /* skip past first field (always empty?) */
     {
-	eventlog(eventlog_level_error,"game_parse_info","bad gameinfo format (missing unknown)");
-	free(save);
+	eventlog(eventlog_level_error,__FUNCTION__,"bad gameinfo format (missing unknown)");
+	xfree(save);
 	return -1;
     }
     if (!(mapsize    = strsep(&currtok,",")))
     {
-	eventlog(eventlog_level_error,"game_parse_info","bad gameinfo format (missing mapsize)");
-	free(save);
+	eventlog(eventlog_level_error,__FUNCTION__,"bad gameinfo format (missing mapsize)");
+	xfree(save);
 	return -1;
     }
     if (!(maxplayers = strsep(&currtok,","))) /* for later use (FIXME: what is upper field, max observers?) */
     {
-	eventlog(eventlog_level_error,"game_parse_info","bad gameinfo format (missing maxplayers)");
-	free(save);
+	eventlog(eventlog_level_error,__FUNCTION__,"bad gameinfo format (missing maxplayers)");
+	xfree(save);
 	return -1;
     }
     if (!(speed      = strsep(&currtok,",")))
     {
-	eventlog(eventlog_level_error,"game_parse_info","bad gameinfo format (missing speed)");
-	free(save);
+	eventlog(eventlog_level_error,__FUNCTION__,"bad gameinfo format (missing speed)");
+	xfree(save);
 	return -1;
     }
     if (!(maptype    = strsep(&currtok,",")))
     {
-	eventlog(eventlog_level_error,"game_parse_info","bad gameinfo format (missing maptype)");
-	free(save);
+	eventlog(eventlog_level_error,__FUNCTION__,"bad gameinfo format (missing maptype)");
+	xfree(save);
 	return -1;
     }
     if (!(gametype   = strsep(&currtok,","))) /* this is set from another field */
     {
-	eventlog(eventlog_level_error,"game_parse_info","bad gameinfo format (missing gametype)");
-	free(save);
+	eventlog(eventlog_level_error,__FUNCTION__,"bad gameinfo format (missing gametype)");
+	xfree(save);
 	return -1;
     }
     if (!(option     = strsep(&currtok,","))) /* this is set from another field */
     {
-	eventlog(eventlog_level_error,"game_parse_info","bad gameinfo format (missing option)");
-	free(save);
+	eventlog(eventlog_level_error,__FUNCTION__,"bad gameinfo format (missing option)");
+	xfree(save);
 	return -1;
     }
     if (!(checksum   = strsep(&currtok,","))) /* FIXME */
     {
-	eventlog(eventlog_level_error,"game_parse_info","bad gameinfo format (missing checksum)");
-	free(save);
+	eventlog(eventlog_level_error,__FUNCTION__,"bad gameinfo format (missing checksum)");
+	xfree(save);
 	return -1;
     }
     if (!(tileset    = strsep(&currtok,",")))
     {
-	eventlog(eventlog_level_error,"game_parse_info","bad gameinfo format (missing tileset)");
-	free(save);
+	eventlog(eventlog_level_error,__FUNCTION__,"bad gameinfo format (missing tileset)");
+	xfree(save);
 	return -1;
     }
     if (!(player     = strsep(&currtok,",")))
     {
-	eventlog(eventlog_level_error,"game_parse_info","bad gameinfo format (missing player)");
-	free(save);
+	eventlog(eventlog_level_error,__FUNCTION__,"bad gameinfo format (missing player)");
+	xfree(save);
 	return -1;
     }
     
     mapname = line2; /* only one item on this line */
     
-    eventlog(eventlog_level_debug,"game_parse_info","got info \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"", mapsize, maxplayers, speed, maptype, gametype, option, checksum, tileset, player, mapname);
+    eventlog(eventlog_level_debug,__FUNCTION__,"got info \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"", mapsize, maxplayers, speed, maptype, gametype, option, checksum, tileset, player, mapname);
     
     /* The map size is determined by breaking the number into two pieces and
      * multiplying each piece by 32.
      * for example, 34 = (32*3) x (32*4) = 96 x 128
      */
     /* special handling for mapsize. empty is 256x256 */
-    if (mapsize[0]!='\0')
-        str_to_uint(mapsize,&bngmapsize);
-    else
+    if ((mapsize[0]=='\0') || (str_to_uint(mapsize,&bngmapsize)<0))
 	bngmapsize = 88; /* 256x256 */
     game_set_mapsize_x(game,(bngmapsize/10)*32);
     game_set_mapsize_y(game,(bngmapsize%10)*32);
     
     /* special handling for maxplayers, empty is 8 */
-    if (maxplayers[0]!='\0')
-	str_to_uint(maxplayers,&bngmaxplayers);
-    else
+    if ((maxplayers[0]=='\0') || (str_to_uint(maxplayers,&bngmaxplayers)<0))
 	bngmaxplayers = 8;
     game_set_maxplayers(game,(bngmaxplayers%10));
     
     /* special handling for gamespeed. empty is fast */
-    if (speed[0]!='\0')
-        str_to_uint(speed,&bngspeed);
-    else
+    if ((speed[0]=='\0') || ( str_to_uint(speed,&bngspeed)<0))
 	bngspeed = CLIENT_GAMESPEED_FAST;
     game_set_speed(game,bngspeed_to_gspeed(bngspeed));
     
     /* special handling for maptype. empty is self-made */
-    if (maptype[0]!='\0')
-        str_to_uint(maptype,&bngmaptype);
-    else
+    if ((maptype[0]=='\0') || (str_to_uint(maptype,&bngmaptype)<0))
 	bngmaptype = CLIENT_MAPTYPE_SELFMADE;
     game_set_maptype(game,bngmaptype_to_gmaptype(bngmaptype));
     
     /* special handling for tileset. empty is badlands */
-    if (tileset[0]!='\0')
-        str_to_uint(tileset,&bngtileset);
-    else
+    if ((tileset[0]=='\0') || (str_to_uint(tileset,&bngtileset)<0))
 	bngtileset = CLIENT_TILESET_BADLANDS;
     game_set_tileset(game,bngtileset_to_gtileset(bngtileset));
     
     game_set_mapname(game,mapname);
     
-    free(save);
+    xfree(save);
     
     return 0;
 }
