@@ -1,4 +1,6 @@
 /*
+ * Copyright (C) 2004 CreepLord (creeplord@pvpgn.org)
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -29,11 +31,6 @@
 #  include <malloc.h>
 # endif
 #endif
-
-#ifdef WIN32_GUI
-# include <win32/winmain.h>
-#endif
-
 #include "common/bn_type.h"
 #include "common/eventlog.h"
 #include "common/packet.h"
@@ -41,14 +38,20 @@
 #include "common/tag.h"
 #include "common/list.h"
 #include "common/util.h"
+#include "common/xalloc.h"
+#include "common/bnettime.h"
 #include "connection.h"
+#include "team.h"
 #include "account.h"
+#include "account_wrap.h"
 #include "channel.h"
 #include "anongame.h"
 #include "anongame_infos.h"
 #include "anongame_maplists.h"
 #include "handle_anongame.h"
 #include "tournament.h"
+#include "server.h"
+#include "clan.h"
 #include "common/setup_after.h"
 
 /* option - handling function */
@@ -61,7 +64,7 @@
 /* 0x05 */ /* AT style search - handle_anongame_search() in anongame.c */
 /* 0x06 */ /* AT style search (Inviter) handle_anongame_search() in anongame.c */
 /* 0x07 */ static int _client_anongame_tournament(t_connection * c, t_packet const * const packet);
-/* 0x08 */ /* unknown if it even exists */
+/* 0x08 */ static int _client_anongame_profile_clan(t_connection * c, t_packet const * const packet);
 /* 0x09 */ static int _client_anongame_get_icon(t_connection * c, t_packet const * const packet);
 /* 0x0A */ static int _client_anongame_set_icon(t_connection * c, t_packet const * const packet);
 
@@ -69,15 +72,90 @@
 static unsigned int _tournament_time_convert(unsigned int time);
 
 /* and now the functions */
+
+static int _client_anongame_profile_clan(t_connection * c, t_packet const * const packet)
+{
+    t_packet * rpacket;
+    int clantag;
+    int clienttag;
+    int count;
+    int temp;
+    t_clan * clan;
+    unsigned char rescount;
+
+    if (packet_get_size(packet)<sizeof(t_client_findanongame_profile_clan))
+    {
+	eventlog(eventlog_level_error,__FUNCTION__,"[%d] got bad ANONGAME_PROFILE_CLAN packet (expected %u bytes, got %u)",conn_get_socket(c), sizeof(t_client_findanongame_profile_clan), packet_get_size(packet));
+	return -1;
+    }
+
+    clantag = bn_int_get(packet->u.client_findanongame_profile_clan.clantag);
+    clienttag = bn_int_get(packet->u.client_findanongame_profile_clan.clienttag);
+    count = bn_int_get(packet->u.server_findanongame_profile_clan.count);
+    clan = clanlist_find_clan_by_clantag(clantag);
+
+    if ((rpacket = packet_create(packet_class_bnet)))
+    {
+	packet_set_size(rpacket,sizeof(t_server_findanongame_profile_clan));
+	packet_set_type(rpacket,SERVER_FINDANONGAME_PROFILE_CLAN);
+	bn_byte_set(&rpacket->u.server_findanongame_profile_clan.option,CLIENT_FINDANONGAME_PROFILE_CLAN);
+	bn_int_set(&rpacket->u.server_findanongame_profile_clan.count,count);
+	rescount = 0;
+
+	if (!(clan))
+	{
+	   temp = 0;
+	   packet_append_data(rpacket,&temp,1);
+	}
+	else
+	{
+	   temp = 0;
+	   packet_append_data(rpacket,&temp,1);
+
+	   /* need to add clan stuff here:
+	   	format:
+		bn_int	ladder_tag (SNLC, 2NLC, 3NLC, 4NLC)
+		bn_int	wins
+		bn_int	losses
+		bn_byte rank
+		bn_byte progess bar
+		bn_int	xp
+		bn_int	rank
+		bn_byte 0x06 <-- random + 5 races
+		6 times:
+		bn_int  wins
+		bn_int	losses
+	    */
+	}
+	
+	bn_byte_set(&rpacket->u.server_findanongame_profile_clan.rescount,rescount);
+
+
+	conn_push_outqueue(c,rpacket);
+	packet_del_ref(rpacket);	
+    }
+   
+   return 0;
+}
+
 static int _client_anongame_profile(t_connection * c, t_packet const * const packet)
 {
     t_packet * rpacket;
     char const * username;
-    int Count;
+    int Count, i;
     int temp;
     t_account * account; 
     t_connection * dest_c;
-    char const * ctag;
+    t_clienttag ctag;
+    char clienttag_str[5];
+    t_list * teamlist;
+    unsigned char teamcount;
+    unsigned char *atcountp;
+    t_elem * curr;
+    t_team * team;
+    t_bnettime bn_time;
+    bn_long ltime;
+    
     
     Count = bn_int_get(packet->u.client_findanongame.count);
     eventlog(eventlog_level_info,__FUNCTION__,"[%d] got a FINDANONGAME PROFILE packet",conn_get_socket(c));
@@ -102,9 +180,12 @@ static int _client_anongame_profile(t_connection * c, t_packet const * const pac
     else
     ctag = conn_get_clienttag(dest_c);
 
-    eventlog(eventlog_level_info,__FUNCTION__,"Looking up %s's WAR3 Stats.",username);
+    eventlog(eventlog_level_info,__FUNCTION__,"Looking up %s's %s Stats.",username,tag_uint_to_str(clienttag_str,ctag));
 
-    if (account_get_sololevel(account,ctag)<=0 && account_get_teamlevel(account,ctag)<=0 && account_get_atteamcount(account,ctag)<=0)
+    if (account_get_ladder_level(account,ctag,ladder_id_solo)<=0 && 
+        account_get_ladder_level(account,ctag,ladder_id_team)<=0 && 
+        account_get_ladder_level(account,ctag,ladder_id_ffa)<=0 && 
+	account_get_teams(account)==NULL)
     {
 	eventlog(eventlog_level_info,__FUNCTION__,"%s does not have WAR3 Stats.",username);
 	if (!(rpacket = packet_create(packet_class_bnet)))
@@ -122,37 +203,37 @@ static int _client_anongame_profile(t_connection * c, t_packet const * const pac
     }
     else // If they do have a profile then:
     {
-	int solowins=account_get_solowin(account,ctag); 
-	int sololoss=account_get_sololoss(account,ctag);
-	int soloxp=account_get_soloxp(account,ctag);
-	int sololevel=account_get_sololevel(account,ctag);
-	int solorank=account_get_solorank(account,ctag);
-	
-	int teamwins=account_get_teamwin(account,ctag);
-	int teamloss=account_get_teamloss(account,ctag);
-	int teamxp=account_get_teamxp(account,ctag);
-	int teamlevel=account_get_teamlevel(account,ctag);
-	int teamrank=account_get_teamrank(account,ctag);
-	
-	int ffawins=account_get_ffawin(account,ctag);
-	int ffaloss=account_get_ffaloss(account,ctag);
-	int ffaxp=account_get_ffaxp(account,ctag);
-	int ffalevel=account_get_ffalevel(account,ctag);
-	int ffarank=account_get_ffarank(account,ctag);
-	
-	int humanwins=account_get_racewin(account,1,ctag);
-	int humanlosses=account_get_raceloss(account,1,ctag);
-	int orcwins=account_get_racewin(account,2,ctag);
-	int orclosses=account_get_raceloss(account,2,ctag);
-	int undeadwins=account_get_racewin(account,8,ctag);
-	int undeadlosses=account_get_raceloss(account,8,ctag);
-	int nightelfwins=account_get_racewin(account,4,ctag);
-	int nightelflosses=account_get_raceloss(account,4,ctag);
-	int randomwins=account_get_racewin(account,0,ctag);
-	int randomlosses=account_get_raceloss(account,0,ctag);
-	
+	int solowins=account_get_ladder_wins(account,ctag,ladder_id_solo); 
+	int sololoss=account_get_ladder_losses(account,ctag,ladder_id_solo);
+	int soloxp=account_get_ladder_xp(account,ctag,ladder_id_solo);
+	int sololevel=account_get_ladder_level(account,ctag,ladder_id_solo);
+	int solorank=account_get_ladder_rank(account,ctag,ladder_id_solo);
+
+	int teamwins=account_get_ladder_wins(account,ctag,ladder_id_team);
+	int teamloss=account_get_ladder_losses(account,ctag,ladder_id_team);
+	int teamxp=account_get_ladder_xp(account,ctag,ladder_id_team);
+	int teamlevel=account_get_ladder_level(account,ctag,ladder_id_team);
+	int teamrank=account_get_ladder_rank(account,ctag,ladder_id_team);
+
+	int ffawins=account_get_ladder_wins(account,ctag,ladder_id_ffa);
+	int ffaloss=account_get_ladder_losses(account,ctag,ladder_id_ffa);
+	int ffaxp=account_get_ladder_xp(account,ctag,ladder_id_ffa);
+	int ffalevel=account_get_ladder_level(account,ctag,ladder_id_ffa);
+	int ffarank=account_get_ladder_rank(account,ctag,ladder_id_ffa);
+
+	int humanwins=account_get_racewins(account,1,ctag);
+	int humanlosses=account_get_racelosses(account,1,ctag);
+	int orcwins=account_get_racewins(account,2,ctag);
+	int orclosses=account_get_racelosses(account,2,ctag);
+	int undeadwins=account_get_racewins(account,8,ctag);
+	int undeadlosses=account_get_racelosses(account,8,ctag);
+	int nightelfwins=account_get_racewins(account,4,ctag);
+	int nightelflosses=account_get_racelosses(account,4,ctag);
+	int randomwins=account_get_racewins(account,0,ctag);
+	int randomlosses=account_get_racelosses(account,0,ctag);
+
 	unsigned char rescount;
-	
+
 	if (!(rpacket = packet_create(packet_class_bnet)))
 	    return -1;
 	packet_set_size(rpacket,sizeof(t_server_findanongame_profile2));
@@ -251,134 +332,72 @@ static int _client_anongame_profile(t_connection * c, t_packet const * const pac
 
 	/* 1 byte team count place holder, set later */
 	packet_append_data(rpacket, &temp, 1);
+	
+	/* we need to store the AT team count but we dont know yet the no
+	 * of stored teams so we cache the pointer for later use 
+	 */
+	atcountp = (unsigned char *)packet_get_raw_data(rpacket, packet_get_size(rpacket) - 1);
 
-	temp=account_get_atteamcount(account,ctag);
-	if(temp>0)
+	teamlist = account_get_teams(account);
+	teamcount = 0;
+	if (teamlist)
 	{
-	    /* [quetzal] 20020827 - partially rewritten AT part */
-	    int i, j, lvl, highest_lvl[6], cnt;
-	    int invalid;
-	    /* allocate array for teamlevels */
-	    int *teamlevels;
-	    unsigned char *atcountp;
-
-	    cnt = temp;
-	    teamlevels = malloc(cnt * sizeof(int));
-
-	    /* we need to store the AT team count but we dont know yet the no
-	     * of corectly stored teams so we cache the pointer for later use 
-	     */
-	    atcountp = (unsigned char *)packet_get_raw_data(rpacket, packet_get_size(rpacket) - 1);
-
-	    /* populate our array */
-	    for (i = 0; i < cnt; i++)
-		if ((teamlevels[i] = account_get_atteamlevel(account, i+1,ctag)) < 0)
-		    teamlevels[i] = 0;
-
-	    /* now lets pick indices of 6 highest levels */
-	    for (j = 0; j < cnt && j < 6; j++) {
-		lvl = -1;
-		for (i = 0; i < cnt; i++) {
-		    if (teamlevels[i] > lvl) {
-			lvl = teamlevels[i];
-			highest_lvl[j] = i;
-		    }
-		}
-		teamlevels[highest_lvl[j]] = -1;
-		eventlog(eventlog_level_debug, __FUNCTION__, 
-		    "profile/AT - highest level is %d with index %d", lvl, highest_lvl[j]);
-	    }
-	    // <-- end of picking indices
-	    // 
-
-	    free((void*)teamlevels);
-
-	    cnt = 0;
-	    invalid = 0;
-	    for(i = 0; i < j; i++)
-	    {
-		int n;
-		int teamsize;
-		char * teammembers, *self, *p2, *p3;
-		int teamtype[] = {0, 0x32565332, 0x33565333, 0x34565334, 0x35565335, 0x36565336};
+	  int teamtype[] = {0, 0x32565332, 0x33565333, 0x34565334, 0x35565335, 0x36565336};
 		
-		n = highest_lvl[i] + 1;
-		teamsize = account_get_atteamsize(account, n, ctag);
+	  LIST_TRAVERSE(teamlist,curr)
+	  {
+	    if (!(team = elem_get_data(curr)))
+	    {
+	      eventlog(eventlog_level_error, __FUNCTION__, "found NULL entry in list");
+	      continue;
+	    }
 
-		teammembers = (char *)account_get_atteammembers(account, n,ctag);
-		if (!teammembers || teamsize < 1 || teamsize > 5) {
-		    eventlog(eventlog_level_warn, __FUNCTION__, "skipping invalid AT (members: '%s' teamsize: %d)", teammembers ? teammembers : "NULL", teamsize);
-		    invalid = 1;
-		    continue;
-		}
+	    if (team_get_clienttag(team) != ctag)
+	      continue;
+	      
+	    bn_int_set((bn_int*)&temp,teamtype[team_get_size(team)-1]);
+	    packet_append_data(rpacket,&temp,4);
 
-		p2 = p3 = teammembers = strdup(teammembers);
-		eventlog(eventlog_level_debug, __FUNCTION__,"profile/AT - processing team %d", n);
+	    bn_int_set((bn_int*)&temp,team_get_wins(team)); //at team wins
+	    packet_append_data(rpacket,&temp,2);
+	    bn_int_set((bn_int*)&temp,team_get_losses(team)); //at team losses
+	    packet_append_data(rpacket,&temp,2);
+	    bn_int_set((bn_int*)&temp,team_get_level(team)); 
+	    packet_append_data(rpacket,&temp,1);
+	    bn_int_set((bn_int*)&temp,account_get_profile_calcs(account,team_get_xp(team),team_get_level(team))); // xp bar calc
+	    packet_append_data(rpacket,&temp,1);
+	    bn_int_set((bn_int*)&temp,team_get_xp(team));
+	    packet_append_data(rpacket,&temp,2);
+	    bn_int_set((bn_int*)&temp,team_get_rank(team)); //rank on AT ladder
+	    packet_append_data(rpacket,&temp,4);
 
-		bn_int_set((bn_int*)&temp,teamtype[teamsize]);
-		packet_append_data(rpacket,&temp,4);
+	    bn_time = time_to_bnettime(temp,team_get_lastgame(team));
+	    bnettime_to_bn_long(bn_time,&ltime);
+	    packet_append_data(rpacket,&ltime,8);
+	  
+	    bn_int_set((bn_int*)&temp,team_get_size(team)-1);
+	    packet_append_data(rpacket,&temp,1);
 
-		bn_int_set((bn_int*)&temp,account_get_atteamwin(account,n,ctag)); //at team wins
-		packet_append_data(rpacket,&temp,2);
-		bn_int_set((bn_int*)&temp,account_get_atteamloss(account,n,ctag)); //at team losses
-		packet_append_data(rpacket,&temp,2);
-		bn_int_set((bn_int*)&temp,account_get_atteamlevel(account,n,ctag)); 
-		packet_append_data(rpacket,&temp,1);
-		bn_int_set((bn_int*)&temp,account_get_profile_calcs(account,account_get_atteamxp(account,n,ctag),account_get_atteamlevel(account,n,ctag))); // xp bar calc
-		packet_append_data(rpacket,&temp,1);
-		bn_int_set((bn_int*)&temp,account_get_atteamxp(account,n,ctag));
-		packet_append_data(rpacket,&temp,2);
-		bn_int_set((bn_int*)&temp,account_get_atteamrank(account,n,ctag)); //rank on AT ladder
-		packet_append_data(rpacket,&temp,4);
-		temp=0;
-		packet_append_data(rpacket,&temp,4); //some unknown packet? random shit
-		packet_append_data(rpacket,&temp,4); //another unknown packet..random shit
-		bn_int_set((bn_int*)&temp,teamsize);
-		packet_append_data(rpacket,&temp,1);
+	    for (i=0; i<team_get_size(team);i++)
+	    {
+	    	if ((team_get_memberuid(team,i)!=account_get_uid(account)))
+		  packet_append_string(rpacket,account_get_name(team_get_member(team,i)));
 		//now attach the names to the packet - not including yourself
 		// [quetzal] 20020826
-		self = strstr(teammembers, account_get_name(account));
 
-		while (p2)
-		{
-		    p3 = strchr(p2, ' ');
-		    if (p3) *p3++ = 0;
-		    if (self != p2) packet_append_string(rpacket, p2);
-		    p2 = p3;
-		}
-
-		free((void *)teammembers);
-		cnt++;
 	    }
+	    teamcount++;
 
-	    if (!cnt) {
-		eventlog(eventlog_level_warn, __FUNCTION__, "no valid team found, sending bogus team");
-		bn_int_set((bn_int*)&temp, 0x32565332);
-		packet_append_data(rpacket,&temp,4);
-
-		temp = 0;
-		packet_append_data(rpacket,&temp,2);
-		packet_append_data(rpacket,&temp,2);
-		packet_append_data(rpacket,&temp,1);
-		packet_append_data(rpacket,&temp,1);
-		packet_append_data(rpacket,&temp,2);
-		packet_append_data(rpacket,&temp,4);
-		packet_append_data(rpacket,&temp,4);
-		packet_append_data(rpacket,&temp,4);
-		bn_int_set((bn_int*)&temp, 1);
-		packet_append_data(rpacket,&temp,1);
-		packet_append_string(rpacket,"error");
-		cnt++;
-	    }
-	    *atcountp = (unsigned char)cnt;
-
-	    if (invalid) account_fix_at(account, ctag);
+	    if ((teamcount>=16)) break;
+	  }
 	}
 
+	*atcountp = (unsigned char)teamcount;
+
 	conn_push_outqueue(c,rpacket);
 	packet_del_ref(rpacket);				
 	
-	eventlog(eventlog_level_info,__FUNCTION__,"Sent %s's WAR3 Stats to requestor.",username);
+	eventlog(eventlog_level_info,__FUNCTION__,"Sent %s's WAR3 Stats (including %d teams) to requestor.",username,teamcount);
     }
     return 0;
 }    
@@ -451,7 +470,7 @@ static int _client_anongame_get_icon(t_connection * c, t_packet const * const pa
         int i,j;
         char rico;
         unsigned int rlvl,rwins;
-	char const * clienttag;
+	t_clienttag clienttag;
 	t_account * acc;
         
         char user_icon[5];
@@ -460,7 +479,7 @@ static int _client_anongame_get_icon(t_connection * c, t_packet const * const pa
 	clienttag = conn_get_clienttag(c);
 	acc = conn_get_account(c);
 	/* WAR3 uses a different table size, might change if blizzard add tournament support to RoC */
-	if (strcmp(clienttag,CLIENTTAG_WARCRAFT3)==0) {
+	if (clienttag==CLIENTTAG_WARCRAFT3_UINT) {
     	    table_width = 5;
 	    table_height= 4;
 	}
@@ -490,10 +509,7 @@ static int _client_anongame_get_icon(t_connection * c, t_packet const * const pa
 	bn_byte_set(&rpacket->u.server_findanongame_iconreply.table_width, table_width);
         bn_byte_set(&rpacket->u.server_findanongame_iconreply.table_size, table_width*table_height);
         for (j=0;j<table_height;j++){
-	    if (strcmp(clienttag,CLIENTTAG_WARCRAFT3)==0)
-		icon_req_race_wins = anongame_infos_get_ICON_REQ_WAR3(j+1);
-	    else
-		icon_req_race_wins = anongame_infos_get_ICON_REQ_W3XP(j+1);
+	    icon_req_race_wins = anongame_infos_get_ICON_REQ(j+1,clienttag);
 	    for (i=0;i<table_width;i++){
 		tempicon.race=i;
 	        tempicon.icon_code[0] = icon_pos[j];
@@ -504,7 +520,7 @@ static int _client_anongame_get_icon(t_connection * c, t_packet const * const pa
 	        if (i<=4){
 	    	    //Building the icon for the races
 	    	    bn_short_set(&tempicon.required_wins,icon_req_race_wins);
-	    	    if (account_get_racewin(acc,race[i],clienttag)>=icon_req_race_wins) {
+	    	    if (account_get_racewins(acc,race[i],clienttag)>=icon_req_race_wins) {
 			tempicon.client_enabled=1;
 	    	    }else{
 			tempicon.client_enabled=0;
@@ -513,7 +529,7 @@ static int _client_anongame_get_icon(t_connection * c, t_packet const * const pa
 	        //Building the icon for the tourney
 		icon_req_tourney_wins = anongame_infos_get_ICON_REQ_TOURNEY(j+1);
 	        bn_short_set(&tempicon.required_wins,icon_req_tourney_wins);
-	        if (account_get_racewin(acc,race[i],clienttag)>=icon_req_tourney_wins) {
+	        if (account_get_racewins(acc,race[i],clienttag)>=icon_req_tourney_wins) {
 		    tempicon.client_enabled=1;
 	        }else{
 		    tempicon.client_enabled=0;}
@@ -550,7 +566,7 @@ static int _client_anongame_set_icon(t_connection * c, t_packet const * const pa
     account_set_user_icon(conn_get_account(c),conn_get_clienttag(c),user_icon);
     //FIXME: Still need a way to 'refresh the user/channel' 
     //_handle_rejoin_command(conn_get_account(c),""); 
-    /* ??? channel_update_flags() */
+    /* ??? channel_update_userflags() */
 	conn_update_w3_playerinfo(c);
     
     channel_rejoin(c);
@@ -582,21 +598,19 @@ static int _client_anongame_infos(t_connection * c, t_packet const * const packe
     } else {
 	int i;
 	int client_tag;
-	int server_tag_count = 0;
+	int server_tag_count	= 0;
 	int client_tag_unk;
 	int server_tag_unk;
 	bn_int temp;
 	char noitems;
 	char * tmpdata;
 	int tmplen;
-	char const * clienttag	= conn_get_clienttag(c);
-	
-	char last_packet		= 0x00;
-	char other_packet		= 0x01;
-    char langstr[5];
-    int gamelang = conn_get_gamelang(c);
-
-    bn_int_tag_get((bn_int const *)&gamelang, langstr, 5);
+	t_clienttag clienttag	= conn_get_clienttag(c);
+	char last_packet	= 0x00;
+	char other_packet	= 0x01;
+	char langstr[5];
+	t_gamelang gamelang = conn_get_gamelang(c);
+	bn_int_tag_get((bn_int const *)&gamelang, langstr, 5);
 	
 	/* Send seperate packet for each item requested
 	 * sending all at once overloaded w3xp
@@ -697,9 +711,8 @@ static int _client_anongame_tournament(t_connection * c, t_packet const * const
     t_packet * rpacket;
     
     t_account * account = conn_get_account(c);
-    char const * clienttag = conn_get_clienttag(c);
+    t_clienttag clienttag = conn_get_clienttag(c);
     
-    unsigned int now		= time(NULL);
     unsigned int start_prelim	= tournament_get_start_preliminary();
     unsigned int end_signup	= tournament_get_end_signup();
     unsigned int end_prelim	= tournament_get_end_preliminary();
@@ -876,29 +889,36 @@ static unsigned int _tournament_time_convert(unsigned int time)
 
 extern int handle_anongame_packet(t_connection * c, t_packet const * const packet)
 {
-    if (bn_byte_get(packet->u.client_anongame.option)==CLIENT_FINDANONGAME_PROFILE)
-	return _client_anongame_profile(c, packet);
+    switch (bn_byte_get(packet->u.client_anongame.option))
+    {
+	case CLIENT_FINDANONGAME_PROFILE:
+	  return _client_anongame_profile(c, packet);
 
-    if (bn_byte_get(packet->u.client_anongame.option) == CLIENT_FINDANONGAME_CANCEL) 
-	return _client_anongame_cancel(c);
+	case CLIENT_FINDANONGAME_CANCEL:
+	  return _client_anongame_cancel(c);
 	
-    if (bn_byte_get(packet->u.client_anongame.option)==CLIENT_FINDANONGAME_SEARCH ||
-	    bn_byte_get(packet->u.client_anongame.option)==CLIENT_FINDANONGAME_AT_INVITER_SEARCH ||
-	    bn_byte_get(packet->u.client_anongame.option)==CLIENT_FINDANONGAME_AT_SEARCH)
-	return handle_anongame_search(c, packet); /* located in anongame.c */
+	case CLIENT_FINDANONGAME_SEARCH:
+	case CLIENT_FINDANONGAME_AT_INVITER_SEARCH:
+	case CLIENT_FINDANONGAME_AT_SEARCH:
+	  return handle_anongame_search(c, packet); /* located in anongame.c */
 
-    if (bn_byte_get(packet->u.client_anongame.option)==CLIENT_FINDANONGAME_GET_ICON)
-	return _client_anongame_get_icon(c, packet);
+	case CLIENT_FINDANONGAME_GET_ICON:
+	  return _client_anongame_get_icon(c, packet);
     
-    if (bn_byte_get(packet->u.client_anongame.option)==CLIENT_FINDANONGAME_SET_ICON)
-	return _client_anongame_set_icon(c, packet);
+	case CLIENT_FINDANONGAME_SET_ICON:
+	  return _client_anongame_set_icon(c, packet);
 
-    if (bn_byte_get(packet->u.client_anongame.option) == CLIENT_FINDANONGAME_INFOS)
-	return _client_anongame_infos(c, packet);
+	case CLIENT_FINDANONGAME_INFOS:
+	  return _client_anongame_infos(c, packet);
     
-    if (bn_byte_get(packet->u.client_anongame.option)==CLIENT_ANONGAME_TOURNAMENT)
-	return _client_anongame_tournament(c, packet);
-    
-    eventlog(eventlog_level_error,__FUNCTION__,"got unhandled option %d",bn_byte_get(packet->u.client_findanongame.option));
-    return -1;
+	case CLIENT_ANONGAME_TOURNAMENT:
+	  return _client_anongame_tournament(c, packet);
+
+	case CLIENT_FINDANONGAME_PROFILE_CLAN:
+	  return _client_anongame_profile_clan(c, packet);
+
+	default:
+          eventlog(eventlog_level_error,__FUNCTION__,"got unhandled option %d",bn_byte_get(packet->u.client_findanongame.option));
+	  return -1;
+    }
 }
