@@ -63,6 +63,8 @@
 #include "message.h"
 #include "prefs.h"
 #include "common/list.h"
+#include "common/xalloc.h"
+#include "server.h"
 #include "ipban.h"
 #include "common/setup_after.h"
 
@@ -84,9 +86,7 @@ static time_t lastchecktime = 0;
 
 extern int ipbanlist_create(void)
 {
-    if (!(ipbanlist_head = list_create()))
-	return -1;
-
+    ipbanlist_head = list_create();
     return 0;
 }
 
@@ -103,11 +103,11 @@ extern int ipbanlist_destroy(void)
 	    entry = elem_get_data(curr);
 	    if (!entry) /* should not happen */
 	    {
-		eventlog(eventlog_level_error,"ipbanlist_destroy","ipbanlist contains NULL item");
+		eventlog(eventlog_level_error,__FUNCTION__,"ipbanlist contains NULL item");
 		continue;
 	    }
-	    if (list_remove_elem(ipbanlist_head,curr)<0)
-		eventlog(eventlog_level_error,"ipbanlist_destroy","could not remove item from list");
+	    if (list_remove_elem(ipbanlist_head,&curr)<0)
+		eventlog(eventlog_level_error,__FUNCTION__,"could not remove item from list");
 	    ipban_unload_entry(entry);
 	}
 	if (list_destroy(ipbanlist_head)<0)
@@ -130,13 +130,13 @@ extern int ipbanlist_load(char const * filename)
     
     if (!filename)
     {
-        eventlog(eventlog_level_error,"ipbanlist_load","got NULL filename");
+        eventlog(eventlog_level_error,__FUNCTION__,"got NULL filename");
 	return -1;
     }
     
     if (!(fp = fopen(filename,"r")))
     {
-        eventlog(eventlog_level_error,"ipbanlist_load","could not open banlist file \"%s\" for reading (fopen: %s)",filename,strerror(errno));
+        eventlog(eventlog_level_error,__FUNCTION__,"could not open banlist file \"%s\" for reading (fopen: %s)",filename,pstrerror(errno));
 	return -1;
     }
 
@@ -148,7 +148,6 @@ extern int ipbanlist_load(char const * filename)
 	while (*ip=='\t' || *ip==' ') ip++;
 	if (*ip=='\0' || *ip=='#')
 	{
-	    free(buff);
 	    continue;
 	}
 	
@@ -172,22 +171,21 @@ extern int ipbanlist_load(char const * filename)
 	else
 	    if (clockstr_to_seconds(timestr,&endtime)<0)
 	    {
-		eventlog(eventlog_level_error,"ipbanlist_load","could not convert to seconds. Banning pernamently.");
+		eventlog(eventlog_level_error,__FUNCTION__,"could not convert to seconds. Banning pernamently.");
 		endtime = 0;
 	    }
 	
 	if (ipbanlist_add(NULL,ip,endtime)!=0)
 	{
-	    free(buff);
-	    eventlog(eventlog_level_warn,"ipbanlist_load","error in %.64s at line %u",filename,currline);
+	    eventlog(eventlog_level_warn,__FUNCTION__,"error in %.64s at line %u",filename,currline);
 	    continue;
 	}
 	
-	free(buff);
     }
     
+    file_get_line(NULL); // clear file_get_line buffer
     if (fclose(fp)<0)
-        eventlog(eventlog_level_error,"ipbanlist_load","could not close banlist file \"%s\" after reading (fclose: %s)",filename,strerror(errno));
+        eventlog(eventlog_level_error,__FUNCTION__,"could not close banlist file \"%s\" after reading (fclose: %s)",filename,pstrerror(errno));
     
     return 0;
 }
@@ -203,18 +201,18 @@ extern int ipbanlist_save(char const * filename)
     
     if (!filename)
     {
-        eventlog(eventlog_level_error,"ipbanlist_save","got NULL filename");
+        eventlog(eventlog_level_error,__FUNCTION__,"got NULL filename");
 	return -1;
     }
     
     if (!(fp = fopen(filename,"w")))
     {
-        eventlog(eventlog_level_error,"ipbanlist_save","could not open banlist file \"%s\" for writing (fopen: %s)",filename,strerror(errno));
+        eventlog(eventlog_level_error,__FUNCTION__,"could not open banlist file \"%s\" for writing (fopen: %s)",filename,pstrerror(errno));
 	return -1;
     }
 /*    if (ftruncate(fp,0)<0)
     {
-        eventlog(eventlog_level_error,"ipbanlist_save","could not truncate banlist file \"%s\" (ftruncate: %s)",filename,strerror(errno));
+        eventlog(eventlog_level_error,__FUNCTION__,"could not truncate banlist file \"%s\" (ftruncate: %s)",filename,pstrerror(errno));
 	return -1;
     }*/
     
@@ -223,12 +221,12 @@ extern int ipbanlist_save(char const * filename)
 	entry = elem_get_data(curr);
 	if (!entry)
 	{
-	    eventlog(eventlog_level_error,"ipbanlist_save","ipbanlist contains NULL element");
+	    eventlog(eventlog_level_error,__FUNCTION__,"ipbanlist contains NULL element");
 	    continue;
 	}
 	if (!(ipstr = ipban_entry_to_str(entry)))
 	{
-	    eventlog(eventlog_level_error,"ipbanlist_save","got NULL ipstr");
+	    eventlog(eventlog_level_error,__FUNCTION__,"got NULL ipstr");
 	    continue;
 	}
 	if (entry->endtime == 0)
@@ -236,13 +234,13 @@ extern int ipbanlist_save(char const * filename)
 	else    
 	    sprintf(line,"%s %ld\n",ipstr,entry->endtime);
 	if (!(fwrite(line,strlen(line),1,fp)))
-	    eventlog(eventlog_level_error,"ipbanlist_save","could not write to banlist file (write: %s)",strerror(errno));
-	free(ipstr);
+	    eventlog(eventlog_level_error,__FUNCTION__,"could not write to banlist file (write: %s)",pstrerror(errno));
+	xfree(ipstr);
     }
 	
     if (fclose(fp)<0)
     {
-        eventlog(eventlog_level_error,"ipbanlist_load","could not close banlist file \"%s\" after writing (fclose: %s)",filename,strerror(errno));
+        eventlog(eventlog_level_error,__FUNCTION__,"could not close banlist file \"%s\" after writing (fclose: %s)",filename,pstrerror(errno));
 	return -1;
     }
     
@@ -260,22 +258,16 @@ extern int ipbanlist_check(char const * ipaddr)
     char const *    ip3;
     char const *    ip4;
     int		    counter;
-    time_t	    now;
     
     if (!ipaddr)
     {
-	eventlog(eventlog_level_warn,"ipban_check","got NULL ipaddr");
+	eventlog(eventlog_level_warn,__FUNCTION__,"got NULL ipaddr");
 	return -1;
     }
     
-    if (!(whole = strdup(ipaddr)))
-    {
-    	eventlog(eventlog_level_warn,"ipban_check","could not allocate memory to check ip against wildcard");
-	return -1;
-    }
-    
-    time(&now);
-    eventlog(eventlog_level_debug,"ipban_check","lastcheck: %u, now: %u, now-lc: %u.",(unsigned)lastchecktime,(unsigned)now,(unsigned)(now-lastchecktime));
+    whole = xstrdup(ipaddr);
+
+    eventlog(eventlog_level_debug,__FUNCTION__,"lastcheck: %u, now: %u, now-lc: %u.",(unsigned)lastchecktime,(unsigned)now,(unsigned)(now-lastchecktime));
     
     if (now - lastchecktime >= (signed)prefs_get_ipban_check_int()) /* unsigned; no need to check prefs < 0 */
     {
@@ -290,12 +282,12 @@ extern int ipbanlist_check(char const * ipaddr)
     
     if (!ip1 || !ip2 || !ip3 || !ip4)
     {
-	eventlog(eventlog_level_warn,"ipban_check","got bad IP address \"%s\"",ipaddr);
-	free(whole);
+	eventlog(eventlog_level_warn,__FUNCTION__,"got bad IP address \"%s\"",ipaddr);
+	xfree(whole);
 	return -1;
     }
     
-    eventlog(eventlog_level_debug,"ipban_check","checking %s.%s.%s.%s",ip1,ip2,ip3,ip4);
+    eventlog(eventlog_level_debug,__FUNCTION__,"checking %s.%s.%s.%s",ip1,ip2,ip3,ip4);
     
     counter = 0;
     LIST_TRAVERSE_CONST(ipbanlist_head,curr)
@@ -303,7 +295,7 @@ extern int ipbanlist_check(char const * ipaddr)
 	entry = elem_get_data(curr);
 	if (!entry)
 	{
-	    eventlog(eventlog_level_error,"ipbanlist_check","ipbanlist contains NULL item");
+	    eventlog(eventlog_level_error,__FUNCTION__,"ipbanlist contains NULL item");
 	    return -1;
 	}
 	counter++; 
@@ -312,48 +304,48 @@ extern int ipbanlist_check(char const * ipaddr)
 	case ipban_type_exact:
 	    if (strcmp(entry->info1,ipaddr)==0)
 	    {
-		eventlog(eventlog_level_debug,"ipbanlist_check","address %s matched exact %s",ipaddr,entry->info1);
-		free(whole);
+		eventlog(eventlog_level_debug,__FUNCTION__,"address %s matched exact %s",ipaddr,entry->info1);
+		xfree(whole);
 		return counter;
 	    }
-	    eventlog(eventlog_level_debug,"ipbanlist_check","address %s does not match exact %s",ipaddr,entry->info1);
+	    eventlog(eventlog_level_debug,__FUNCTION__,"address %s does not match exact %s",ipaddr,entry->info1);
 	    continue;
 	 
 	case ipban_type_wildcard:
 	    if (strcmp(entry->info1,"*")!=0 && strcmp(ip1,entry->info1)!=0)
 	    {
-		eventlog(eventlog_level_debug,"ipbanlist_check","address %s does not match part 1 of wildcard %s.%s.%s.%s",ipaddr,entry->info1,entry->info2,entry->info3,entry->info4);
+		eventlog(eventlog_level_debug,__FUNCTION__,"address %s does not match part 1 of wildcard %s.%s.%s.%s",ipaddr,entry->info1,entry->info2,entry->info3,entry->info4);
 		continue;
 	    }
 	    if (strcmp(entry->info2,"*")!=0 && strcmp(ip2,entry->info2)!=0)
 	    {
-		eventlog(eventlog_level_debug,"ipbanlist_check","address %s does not match part 2 of wildcard %s.%s.%s.%s",ipaddr,entry->info1,entry->info2,entry->info3,entry->info4);
+		eventlog(eventlog_level_debug,__FUNCTION__,"address %s does not match part 2 of wildcard %s.%s.%s.%s",ipaddr,entry->info1,entry->info2,entry->info3,entry->info4);
 		continue;
 	    }
 	    if (strcmp(entry->info3,"*")!=0 && strcmp(ip3,entry->info3)!=0)
 	    {
-		eventlog(eventlog_level_debug,"ipbanlist_check","address %s does not match part 3 of wildcard %s.%s.%s.%s",ipaddr,entry->info1,entry->info2,entry->info3,entry->info4);
+		eventlog(eventlog_level_debug,__FUNCTION__,"address %s does not match part 3 of wildcard %s.%s.%s.%s",ipaddr,entry->info1,entry->info2,entry->info3,entry->info4);
 		continue;
 	    }
 	    if (strcmp(entry->info4,"*")!=0 && strcmp(ip4,entry->info4)!=0)
 	    {
-		eventlog(eventlog_level_debug,"ipbanlist_check","address %s does not match part 4 of wildcard %s.%s.%s.%s",ipaddr,entry->info1,entry->info2,entry->info3,entry->info4);
+		eventlog(eventlog_level_debug,__FUNCTION__,"address %s does not match part 4 of wildcard %s.%s.%s.%s",ipaddr,entry->info1,entry->info2,entry->info3,entry->info4);
 		continue;
 	    }
 	    
-	    eventlog(eventlog_level_debug,"ipbanlist_check","address %s matched wildcard %s.%s.%s.%s",ipaddr,entry->info1,entry->info2,entry->info3,entry->info4);
-	    free(whole);
+	    eventlog(eventlog_level_debug,__FUNCTION__,"address %s matched wildcard %s.%s.%s.%s",ipaddr,entry->info1,entry->info2,entry->info3,entry->info4);
+	    xfree(whole);
 	    return counter;
 	    
 	case ipban_type_range:
 	    if ((ipban_str_to_ulong(ipaddr) >= ipban_str_to_ulong(entry->info1)) &&
 		(ipban_str_to_ulong(ipaddr) <= ipban_str_to_ulong(entry->info2)))
 	    {
-		eventlog(eventlog_level_debug,"ipbanlist_check","address %s matched range %s-%s",ipaddr,entry->info1,entry->info2);
-		free(whole);
+		eventlog(eventlog_level_debug,__FUNCTION__,"address %s matched range %s-%s",ipaddr,entry->info1,entry->info2);
+		xfree(whole);
 		return counter;
 	    }
-	    eventlog(eventlog_level_debug,"ipbanlist_check","address %s does not match range %s-%s",ipaddr,entry->info1,entry->info2);
+	    eventlog(eventlog_level_debug,__FUNCTION__,"address %s does not match range %s-%s",ipaddr,entry->info1,entry->info2);
 	    continue;
 	
 	case ipban_type_netmask:
@@ -373,11 +365,11 @@ extern int ipbanlist_check(char const * ipaddr)
 		lip2 = lip2 & netmask;		
 		if (lip1 == lip2)
 		{
-		    eventlog(eventlog_level_debug,"ipbanlist_check","address %s matched netmask %s/%s",ipaddr,entry->info1,entry->info2);
-		    free(whole);
+		    eventlog(eventlog_level_debug,__FUNCTION__,"address %s matched netmask %s/%s",ipaddr,entry->info1,entry->info2);
+		    xfree(whole);
 		    return counter;
 		}
-		eventlog(eventlog_level_debug,"ipbanlist_check","address %s does not match netmask %s/%s",ipaddr,entry->info1,entry->info2);
+		eventlog(eventlog_level_debug,__FUNCTION__,"address %s does not match netmask %s/%s",ipaddr,entry->info1,entry->info2);
 		continue;
 	    }	
 	
@@ -397,19 +389,19 @@ extern int ipbanlist_check(char const * ipaddr)
 		lip2 = lip2 >> (32 - prefix);		
 		if (lip1 == lip2)
 		{
-		    eventlog(eventlog_level_debug,"ipbanlist_check","address %s matched prefix %s/%s",ipaddr,entry->info1,entry->info2);
-		    free(whole);
+		    eventlog(eventlog_level_debug,__FUNCTION__,"address %s matched prefix %s/%s",ipaddr,entry->info1,entry->info2);
+		    xfree(whole);
 		    return counter;
 		}
-		eventlog(eventlog_level_debug,"ipbanlist_check","address %s does not match prefix %s/%s",ipaddr,entry->info1,entry->info2);
+		eventlog(eventlog_level_debug,__FUNCTION__,"address %s does not match prefix %s/%s",ipaddr,entry->info1,entry->info2);
 		continue;
 	    }	
 	default:  /* unknown type */
-	    eventlog(eventlog_level_warn,"ipbanlist_check","found bad ban type %d",(int)entry->type);
+	    eventlog(eventlog_level_warn,__FUNCTION__,"found bad ban type %d",(int)entry->type);
 	}
     }    
 
-    free(whole);
+    xfree(whole);
 
     return 0;
 }
@@ -424,38 +416,28 @@ extern int ipbanlist_add(t_connection * c, char const * cp, time_t endtime)
     {
 	if (c)
 	    message_send_text(c,message_type_error,c,"Bad IP.");
-        eventlog(eventlog_level_error,"ipbanlist_add","could not convert to t_ipban_entry: \"%s\"",cp);
+        eventlog(eventlog_level_error,__FUNCTION__,"could not convert to t_ipban_entry: \"%s\"",cp);
         return -1;
     }
 
     entry->endtime = endtime;
-    if (list_append_data(ipbanlist_head,entry)<0)
-    {
-	ipban_unload_entry(entry);
-	eventlog(eventlog_level_error,"ipbanlist_add","could not append entry");
-	return -1;
-    }
+    list_append_data(ipbanlist_head,entry);
     
     if (c)
     {
-	char const *	tname;
-	time_t		now;
 	
-	time(&now);
 	if (endtime == 0)
 	{
-            sprintf(tstr,"%s banned permamently by %s.",cp,(tname = conn_get_username(c)));
-	    conn_unget_username(c,tname);
-            eventlog(eventlog_level_info,"ipbanlist_add",tstr);
+            sprintf(tstr,"%s banned permamently by %s.",cp,conn_get_username(c));
+            eventlog(eventlog_level_info,__FUNCTION__,tstr);
             message_send_admins(c,message_type_info,tstr);
 	    sprintf(tstr,"%s banned permamently.",cp);
 	    message_send_text(c,message_type_info,c,tstr);
 	}
 	else
 	{
-            sprintf(tstr,"%s banned for %.48s by %s.",cp,seconds_to_timestr(entry->endtime - now),(tname = conn_get_username(c)));
-	    conn_unget_username(c,tname);
-            eventlog(eventlog_level_info,"ipbanlist_add",tstr);
+            sprintf(tstr,"%s banned for %.48s by %s.",cp,seconds_to_timestr(entry->endtime - now),conn_get_username(c));
+            eventlog(eventlog_level_info,__FUNCTION__,tstr);
             message_send_admins(c,message_type_info,tstr);
 	    sprintf(tstr,"%s banned for %.48s.",cp,seconds_to_timestr(entry->endtime - now));
 	    message_send_text(c,message_type_info,c,tstr);
@@ -470,30 +452,27 @@ extern int ipbanlist_unload_expired(void)
 {
     t_elem *		curr;
     t_ipban_entry * 	entry;
-    time_t		now;
     char removed;
     
-    time(&now);
     removed = 0;
     LIST_TRAVERSE(ipbanlist_head,curr)
     {
 	entry = elem_get_data(curr);
 	if (!entry)
 	{
-	    eventlog(eventlog_level_error,"ipbanlist_unload_expired","ipbanlist_contains NULL element");
+	    eventlog(eventlog_level_error,__FUNCTION__,"ipbanlist_contains NULL element");
 	    return -1;
 	}
 	if ((entry->endtime - now <= 0) && (entry->endtime != 0))
 	{
-	    eventlog(eventlog_level_debug,"ipbanlist_unload_expired","removing item: %s",entry->info1);
+	    eventlog(eventlog_level_debug,__FUNCTION__,"removing item: %s",entry->info1);
 	    removed = 1;
-	    if (list_remove_elem(ipbanlist_head,curr)<0)
-		eventlog(eventlog_level_error,"ipbanlist_unload_expired","could not remove item");
+	    if (list_remove_elem(ipbanlist_head,&curr)<0)
+		eventlog(eventlog_level_error,__FUNCTION__,"could not remove item");
 	    else
 		ipban_unload_entry(entry);
 	}
     }
-    list_purge(ipbanlist_head);
     if (removed==1) ipbanlist_save(prefs_get_ipbanfile());
     return 0;
 }
@@ -505,9 +484,8 @@ extern time_t ipbanlist_str_to_time_t(t_connection * c, char const * timestr)
     char		minstr[MAX_TIME_STR];
     unsigned int	i;
     char		tstr[MAX_MESSAGE_LEN];
-    time_t		now;
     
-    for (i=0; isdigit(timestr[i]) && i<sizeof(minstr)-1; i++)
+    for (i=0; isdigit((int)timestr[i]) && i<sizeof(minstr)-1; i++)
 	minstr[i] = timestr[i];
     minstr[i] = '\0';
 
@@ -527,14 +505,13 @@ extern time_t ipbanlist_str_to_time_t(t_connection * c, char const * timestr)
 
     if (clockstr_to_seconds(minstr,&bmin)<0) /* it thinks these are seconds but we treat them as minutes */
     {
-        eventlog(eventlog_level_error,"ipbanlist_str_to_time_t","could not convert to minutes: \"%s\"",timestr);
+        eventlog(eventlog_level_error,__FUNCTION__,"could not convert to minutes: \"%s\"",timestr);
         return -1;
     }
     if (bmin == 0)
     	return 0;
     else
     {	
-	time(&now);
 	return now + bmin*60;
     }
 }
@@ -627,21 +604,20 @@ static int ipban_func_del(t_connection * c, char const * cp)
 	    entry = elem_get_data(curr);
 	    if (!entry)
 	    {
-		eventlog(eventlog_level_error,"ipban_func_del","ipbanlist contains NULL item");
+		eventlog(eventlog_level_error,__FUNCTION__,"ipbanlist contains NULL item");
 		return -1;
 	    }
 	    if (ipban_identical_entry(to_delete,entry))
 	    {
 		counter++;
-		if (list_remove_elem(ipbanlist_head,curr)<0)
-		    eventlog(eventlog_level_error,"ipbanlist_unload_expired","could not remove item");
+		if (list_remove_elem(ipbanlist_head,&curr)<0)
+		    eventlog(eventlog_level_error,__FUNCTION__,"could not remove item");
 		else
 		    ipban_unload_entry(entry);
 	    }
 	}
 	    
 	ipban_unload_entry(to_delete);
-	list_purge(ipbanlist_head);
 	if (counter == 0)
 	{
 	    message_send_text(c,message_type_error,c,"No matching entry.");
@@ -671,11 +647,11 @@ static int ipban_func_del(t_connection * c, char const * cp)
 	    entry = elem_get_data(curr);
 	    if (!entry)
 	    {
-		eventlog(eventlog_level_error,"ipban_func_del","ipbanlist contains NULL item");
+		eventlog(eventlog_level_error,__FUNCTION__,"ipbanlist contains NULL item");
 		return -1;
 	    }
-	    if (list_remove_elem(ipbanlist_head,curr)<0)
-	        eventlog(eventlog_level_error,"ipbanlist_unload_expired","could not remove item");
+	    if (list_remove_elem(ipbanlist_head,&curr)<0)
+	        eventlog(eventlog_level_error,__FUNCTION__,"could not remove item");
 	    else
 	    {
 	        ipban_unload_entry(entry);
@@ -684,7 +660,6 @@ static int ipban_func_del(t_connection * c, char const * cp)
 	}
     }
     
-    list_purge(ipbanlist_head);
     if (to_delete_nmbr > counter)
     {
 	sprintf(tstr,"There are only %u entries.",counter);
@@ -703,10 +678,8 @@ static int ipban_func_list(t_connection * c)
     char		tstr[MAX_MESSAGE_LEN];
     unsigned int	counter;
     char	 	timestr[50];
-    time_t		now;
     char *		ipstr;
     
-    time(&now);
     counter = 0;
     message_send_text(c,message_type_info,c,"Banned IPs:");
     LIST_TRAVERSE_CONST(ipbanlist_head,curr)
@@ -714,7 +687,7 @@ static int ipban_func_list(t_connection * c)
 	entry = elem_get_data(curr);
 	if (!entry)
 	{
-	    eventlog(eventlog_level_error,"ipban_func_list","ipbanlist contains NULL item");
+	    eventlog(eventlog_level_error,__FUNCTION__,"ipbanlist contains NULL item");
 	    return -1;
 	}
 	counter++;
@@ -725,12 +698,12 @@ static int ipban_func_list(t_connection * c)
 
 	if (!(ipstr = ipban_entry_to_str(entry)))
 	{
-	    eventlog(eventlog_level_error,"ipban_func_list","could not convert entry to string");
+	    eventlog(eventlog_level_error,__FUNCTION__,"could not convert entry to string");
 	    continue;
 	}
 	sprintf(tstr,"%u: %s %s",counter,ipstr,timestr);
 	message_send_text(c,message_type_info,c,tstr);
-	free(ipstr);
+	xfree(ipstr);
     }
 
     if (counter == 0)
@@ -793,7 +766,7 @@ static int ipban_identical_entry(t_ipban_entry * e1, t_ipban_entry * e2)
 		return 1;
 	    break;
         default:  /* unknown type */
-	    eventlog(eventlog_level_warn,"ipbanlist_identical_entry","found bad ban type %d",(int)e2->type);
+	    eventlog(eventlog_level_warn,__FUNCTION__,"found bad ban type %d",(int)e2->type);
     }
 
     return 0;
@@ -807,21 +780,21 @@ static int ipban_unload_entry(t_ipban_entry * e)
     	case ipban_type_exact:
 	case ipban_type_wildcard:
 	    if (e->info1)
-		free(e->info1);
+		xfree(e->info1);
 	    break;
 	case ipban_type_range:
 	case ipban_type_netmask:
 	case ipban_type_prefix:
 	    if (e->info1)
-		free(e->info1);
+		xfree(e->info1);
 	    if (e->info2)
-		free(e->info2);
+		xfree(e->info2);
 	    break;
 	default:  /* unknown type */
-    	    eventlog(eventlog_level_warn,"ipbanlist_unload","found bad ban type %d",(int)e->type);
+    	    eventlog(eventlog_level_warn,__FUNCTION__,"found bad ban type %d",(int)e->type);
 	    return -1;
     }
-    free(e);
+    xfree(e);
     return 0;
 }
 
@@ -833,56 +806,34 @@ static t_ipban_entry * ipban_str_to_ipban_entry(char const * ipstr)
     char *          cp;
     t_ipban_entry * entry;
     
-    if (!(entry = malloc(sizeof(t_ipban_entry))))
-    {
-        eventlog(eventlog_level_error,"ipban_str_to_t_ipban_entry","could not allocate memory for entry %s", ipstr);
-        return NULL;
-    }
+    entry = xmalloc(sizeof(t_ipban_entry));
     if (!ipstr)
     {
-	eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","got NULL IP");
-	free(entry);
+	eventlog(eventlog_level_error,__FUNCTION__,"got NULL IP");
+	xfree(entry);
 	return NULL;
     }
     if (ipstr[0] == '\0')
     {
-        eventlog(eventlog_level_warn,"ipban_str_to_ipban_entry","got empty IP string");
-	free(entry);
+        eventlog(eventlog_level_warn,__FUNCTION__,"got empty IP string");
+	xfree(entry);
         return NULL;
     }
-    if (!(cp = strdup(ipstr)))
-    {
-	eventlog(eventlog_level_warn,"ipban_str_to_ipban_entry","could not allocate memory for cp");
-	free(entry);
-	return NULL;
-    }
+    cp = xstrdup(ipstr);
 
     if (ipban_could_be_ip_str(cp)==0)
     {
-	eventlog(eventlog_level_debug,"ipban_str_to_ipban_entry","string: \"%.32s\" can not be valid IP",cp);
-	free(entry);
+	eventlog(eventlog_level_debug,__FUNCTION__,"string: \"%.32s\" can not be valid IP",cp);
+	xfree(entry);
 	return NULL;
     }
     if ((matched = strchr(cp,'-'))) /* range */
     {
         entry->type = ipban_type_range;
-	eventlog(eventlog_level_debug,"ipban_str_to_ipban_entry","entry: %s matched as ipban_type_range",cp);
+	eventlog(eventlog_level_debug,__FUNCTION__,"entry: %s matched as ipban_type_range",cp);
         matched[0] = '\0';
-        if (!(entry->info1 = strdup(cp))) /* start of range */
-        {
-	    eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","could not allocate memory for info on entry %s",cp);
-	    free(entry);
-	    free(cp);
-	    return NULL;
-	}
-	if (!(entry->info2 = strdup(&matched[1]))) /* end of range */
-	{
-	    eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","could not allocate memory for info on entry %s",cp);
-	    free(entry->info1);
-	    free(entry);
-	    free(cp);
-	    return NULL;
-	}
+        entry->info1 = xstrdup(cp); /* start of range */
+	entry->info2 = xstrdup(&matched[1]); /* end of range */
 	entry->info3 = NULL; /* clear unused elements so debugging is nicer */
 	entry->info4 = NULL;
     }
@@ -890,26 +841,20 @@ static t_ipban_entry * ipban_str_to_ipban_entry(char const * ipstr)
         if (strchr(cp,'*')) /* wildcard */
         {
 	    entry->type = ipban_type_wildcard;
-	    eventlog(eventlog_level_debug,"ipban_str_to_ipban_entry","entry: %s matched as ipban_type_wildcard",cp);
+	    eventlog(eventlog_level_debug,__FUNCTION__,"entry: %s matched as ipban_type_wildcard",cp);
 		
-	    /* only free() info1! */
-	    if (!(whole = strdup(cp)))
-	    {
-	        eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","could not allocate memory for info on entry \"%s\"",cp);
-	        free(entry);
-		free(cp);
-	        return NULL;
-	    }
+	    /* only xfree() info1! */
+	    whole = xstrdup(cp);
 	    entry->info1 = strtok(whole,".");
 	    entry->info2 = strtok(NULL,".");
 	    entry->info3 = strtok(NULL,".");
 	    entry->info4 = strtok(NULL,".");
 	    if (!entry->info4) /* not enough dots */
 	    {
-	        eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","wildcard entry \"%s\" does not contain all four octets",cp);
-	        free(entry->info1);
-	        free(entry);
-		free(cp);
+	        eventlog(eventlog_level_error,__FUNCTION__,"wildcard entry \"%s\" does not contain all four octets",cp);
+	        xfree(entry->info1);
+	        xfree(entry);
+		xfree(cp);
 	        return NULL;
 	    }
 	}
@@ -919,50 +864,31 @@ static t_ipban_entry * ipban_str_to_ipban_entry(char const * ipstr)
 		if (strchr(&matched[1],'.'))
 		{
 		    entry->type = ipban_type_netmask;
-		    eventlog(eventlog_level_debug,"ipban_str_to_ipban_entry","entry: %s matched as ipban_type_netmask",cp);
+		    eventlog(eventlog_level_debug,__FUNCTION__,"entry: %s matched as ipban_type_netmask",cp);
 		}
 		else
 		{
 		    entry->type = ipban_type_prefix;
-		    eventlog(eventlog_level_debug,"ipban_str_to_ipban_entry","entry: %s matched as ipban_type_prefix",cp);
+		    eventlog(eventlog_level_debug,__FUNCTION__,"entry: %s matched as ipban_type_prefix",cp);
 		}
 	    
 		matched[0] = '\0';
-		if (!(entry->info1 = strdup(cp)))
-		{
-		    eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","could not allocate memory for info on entry \"%s\"",cp);
-	    	    free(entry);
-		    free(cp);
-		    return NULL;
-		}
-		if (!(entry->info2 = strdup(&matched[1])))
-		{
-		    eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","could not allocate memory for info on entry \"%s\"",cp);
-		    free(entry->info1);
-		    free(entry);
-		    free(cp);
-		    return NULL;
-		}
+		entry->info1 = xstrdup(cp);
+		entry->info2 = xstrdup(&matched[1]);
 		entry->info3 = NULL; /* clear unused elements so debugging is nicer */
 		entry->info4 = NULL;
 	    }
 	    else /* exact */
 	    {
 		entry->type = ipban_type_exact;
-		eventlog(eventlog_level_debug,"ipban_str_to_ipban_entry","entry: %s matched as ipban_type_exact",cp);
+		eventlog(eventlog_level_debug,__FUNCTION__,"entry: %s matched as ipban_type_exact",cp);
 	        
-		if (!(entry->info1 = strdup(cp)))
-		{
-		    eventlog(eventlog_level_error,"ipban_str_to_ipban_entry","could not allocate memory for info on entry \"%s\"",cp);
-		    free(entry);
-		    free(cp);
-		    return NULL;
-		}
+		entry->info1 = xstrdup(cp);
 		entry->info2 = NULL; /* clear unused elements so debugging is nicer */
 		entry->info3 = NULL;
 		entry->info4 = NULL;
 	    }
-    free(cp);
+    xfree(cp);
     
     return entry;
 }
@@ -990,14 +916,10 @@ static char * ipban_entry_to_str(t_ipban_entry const * entry)
 	    break;
 	    
 	default: /* unknown type */
-	    eventlog(eventlog_level_warn,"ipban_entry_to_str","found bad ban type %d",(int)entry->type);
+	    eventlog(eventlog_level_warn,__FUNCTION__,"found bad ban type %d",(int)entry->type);
 	    return NULL;
     }
-    if (!(str = strdup(tstr)))
-    {
-	eventlog(eventlog_level_error,"ipban_entry_to_str","could not allocate memmory for str");
-	return NULL;
-    }
+    str = xstrdup(tstr);
     
     return str;
 }
@@ -1011,18 +933,14 @@ static unsigned long ipban_str_to_ulong(char const * ipaddr)
     char *    		ip4;
     char *		tipaddr;
     
-    if (!(tipaddr = strdup(ipaddr)))
-    {
-        eventlog(eventlog_level_warn,"ipban_ipstr_to_ulong","could not allocate memory for tipaddr");
-	return 0;
-    }
+    tipaddr = xstrdup(ipaddr);
     ip1 = strtok(tipaddr,".");
     ip2 = strtok(NULL,".");
     ip3 = strtok(NULL,".");
     ip4 = strtok(NULL,".");
     lip = (atoi(ip1) << 24) + (atoi(ip2) << 16) + (atoi(ip3) << 8) + (atoi(ip4));
 
-    free(tipaddr);
+    xfree(tipaddr);
 
     return lip;
 }
@@ -1035,11 +953,7 @@ static int ipban_could_be_exact_ip_str(char const * str)
     char * 	ttok;
     int 	i;
     
-    if (!(ipstr = strdup(str)))
-    {
-	eventlog(eventlog_level_error,"ipban_could_be_exact_ip_str","could not allocate memmory for ipstr");
-	return 0;
-    }
+    ipstr = xstrdup(str);
     
     s = ipstr;
     for (i=0; i<4; i++)
@@ -1047,12 +961,12 @@ static int ipban_could_be_exact_ip_str(char const * str)
 	ttok = (char *)strsep(&ipstr, ".");
 	if (!ttok || strlen(ttok)<1 || strlen(ttok)>3)
 	{
-	    free(s);
+	    xfree(s);
 	    return 0;
 	}
     }
 
-    free(s);
+    xfree(s);
     return 1;
 }
 
@@ -1065,28 +979,24 @@ static int ipban_could_be_ip_str(char const * str)
     
     if (strlen(str)<7)
     {
-	eventlog(eventlog_level_error,"ipban_could_be_ip_str","string too short");
+	eventlog(eventlog_level_error,__FUNCTION__,"string too short");
 	return 0;
     }
     for (i=0; i<strlen(str); i++)
-	if (!isdigit(str[i]) && str[i]!='.' && str[i]!='*' && str[i]!='/' && str[i]!='-')
+	if (!isdigit((int)str[i]) && str[i]!='.' && str[i]!='*' && str[i]!='/' && str[i]!='-')
 	{
-	    eventlog(eventlog_level_debug,"ipban_could_be_ip_str","illegal character on position %i",i);
+	    eventlog(eventlog_level_debug,__FUNCTION__,"illegal character on position %i",i);
 	    return 0;
 	}
 
-    if (!(ipstr = strdup(str)))
-    {
-	eventlog(eventlog_level_error,"ipban_could_be_ip_str","could not allocate memory for ipstr");
-	return 0;
-    }
+    ipstr = xstrdup(str);
     if ((matched = strchr(ipstr,'-')))
     {
 	matched[0] = '\0';
 	if ((ipban_could_be_exact_ip_str(ipstr)==0) ||
 	    (ipban_could_be_exact_ip_str(&matched[1])==0))
 	{
-	    free(ipstr);
+	    xfree(ipstr);
 	    return 0;
 	}
     }
@@ -1094,7 +1004,7 @@ static int ipban_could_be_ip_str(char const * str)
     {
 	if (ipban_could_be_exact_ip_str(ipstr)==0) /* FIXME: 123.123.1*.123 allowed */
 	{
-	    free(ipstr);
+	    xfree(ipstr);
 	    return 0;
 	}
     }
@@ -1106,7 +1016,7 @@ static int ipban_could_be_ip_str(char const * str)
 	    if ((ipban_could_be_exact_ip_str(ipstr)==0) ||
 		(ipban_could_be_exact_ip_str(&matched[1])==0))
 	    {
-		free(ipstr);
+		xfree(ipstr);
 		return 0;
 	    }
 	}
@@ -1114,18 +1024,18 @@ static int ipban_could_be_ip_str(char const * str)
 	{
     	    if (ipban_could_be_exact_ip_str(ipstr)==0)
 	    {
-		free(ipstr);
+		xfree(ipstr);
 		return 0;
 	    }
 	    for (i=1; i<strlen(&matched[1]); i++)
-		if (!isdigit(matched[i]))
+		if (!isdigit((int)matched[i]))
 		{
-		    free(ipstr);
+		    xfree(ipstr);
 		    return 0;
 		}
 	    if (atoi(&matched[1])>32) /* can not be less than 0 because IP/-24 is matched as range */
 	    {
-		free(ipstr);
+		xfree(ipstr);
 		return 0;
 	    }
 	}
@@ -1134,11 +1044,11 @@ static int ipban_could_be_ip_str(char const * str)
     {
 	if (ipban_could_be_exact_ip_str(ipstr)==0)
 	{
-	    free(ipstr);
+	    xfree(ipstr);
 	    return 0;
 	}
     }
-    free(ipstr);
+    xfree(ipstr);
     
     return 1;    
 }
