@@ -86,6 +86,7 @@
 #include "common/eventlog.h"
 #include "common/util.h"
 #include "common/tracker.h"
+#include "common/xalloc.h"
 #include "common/setup_after.h"
 
 
@@ -148,7 +149,7 @@ extern int main(int argc, char * argv[])
 	eventlog_set(stderr);
         if (eventlog_open(prefs.logfile)<0)
 	{
-            eventlog(eventlog_level_fatal,"main","could not use file \"%s\" for the eventlog (exiting)",prefs.logfile);
+            eventlog(eventlog_level_fatal,__FUNCTION__,"could not use file \"%s\" for the eventlog (exiting)",prefs.logfile);
 	    return STATUS_FAILURE;
 	}
     }
@@ -159,7 +160,7 @@ extern int main(int argc, char * argv[])
 	switch (fork())
 	{
 	case -1:
-	    eventlog(eventlog_level_error,"main","could not fork (fork: %s)\n",strerror(errno));
+	    eventlog(eventlog_level_error,__FUNCTION__,"could not fork (fork: %s)\n",pstrerror(errno));
 	    return STATUS_FAILURE;
 	case 0: /* child */
 	    break;
@@ -174,7 +175,7 @@ extern int main(int argc, char * argv[])
 # ifdef HAVE_SETPGID
 	if (setpgid(0,0)<0)
 	{
-	    eventlog(eventlog_level_error,"main","could not create new process group (setpgid: %s)\n",strerror(errno));
+	    eventlog(eventlog_level_error,__FUNCTION__,"could not create new process group (setpgid: %s)\n",pstrerror(errno));
 	    return STATUS_FAILURE;
 	}
 # else
@@ -182,13 +183,13 @@ extern int main(int argc, char * argv[])
 #   ifdef SETPGRP_VOID
 	if (setpgrp()<0)
 	{
-	    eventlog(eventlog_level_error,"main","could not create new process group (setpgrp: %s)\n",strerror(errno));
+	    eventlog(eventlog_level_error,__FUNCTION__,"could not create new process group (setpgrp: %s)\n",pstrerror(errno));
 	    return STATUS_FAILURE;
 	}
 #   else
 	if (setpgrp(0,0)<0)
 	{
-	    eventlog(eventlog_level_error,"main","could not create new process group (setpgrp: %s)\n",strerror(errno));
+	    eventlog(eventlog_level_error,__FUNCTION__,"could not create new process group (setpgrp: %s)\n",pstrerror(errno));
 	    return STATUS_FAILURE;
 	}
 #   endif
@@ -196,7 +197,7 @@ extern int main(int argc, char * argv[])
 #   ifdef HAVE_SETSID
 	if (setsid()<0)
 	{
-	    eventlog(eventlog_level_error,"main","could not create new process group (setsid: %s)\n",strerror(errno));
+	    eventlog(eventlog_level_error,__FUNCTION__,"could not create new process group (setsid: %s)\n",pstrerror(errno));
 	    return STATUS_FAILURE;
 	}
 #   else
@@ -214,37 +215,37 @@ extern int main(int argc, char * argv[])
 
         if (!(fp = fopen(prefs.pidfile,"w")))
         {
-            eventlog(eventlog_level_error,"main","unable to open pid file \"%s\" for writing (fopen: %s)",prefs.pidfile,strerror(errno));
+            eventlog(eventlog_level_error,__FUNCTION__,"unable to open pid file \"%s\" for writing (fopen: %s)",prefs.pidfile,pstrerror(errno));
             prefs.pidfile = NULL;
         }
         else
         {
             fprintf(fp,"%u",(unsigned int)getpid());
             if (fclose(fp)<0)
-                eventlog(eventlog_level_error,"main","could not close pid file \"%s\" after writing (fclose: %s)",prefs.pidfile,strerror(errno));
+                eventlog(eventlog_level_error,__FUNCTION__,"could not close pid file \"%s\" after writing (fclose: %s)",prefs.pidfile,pstrerror(errno));
         }
 #else
-        eventlog(eventlog_level_warn,"main","no getpid() system call, do not use the -P or the --pidfile option");
+        eventlog(eventlog_level_warn,__FUNCTION__,"no getpid() system call, do not use the -P or the --pidfile option");
         prefs.pidfile = NULL;
 #endif
     }
     
 #ifdef HAVE_GETPID
-    eventlog(eventlog_level_info,"main","bntrackd version "PVPGN_VERSION" process %u",(unsigned int)getpid());
+    eventlog(eventlog_level_info,__FUNCTION__,"bntrackd version "PVPGN_VERSION" process %u",(unsigned int)getpid());
 #else
-    eventlog(eventlog_level_info,"main","bntrackd version "PVPGN_VERSION);
+    eventlog(eventlog_level_info,__FUNCTION__,"bntrackd version "PVPGN_VERSION);
 #endif
     
     if (psock_init()<0)
     {
-        eventlog(eventlog_level_error,"main","could not initialize socket functions");
+        eventlog(eventlog_level_error,__FUNCTION__,"could not initialize socket functions");
         return STATUS_FAILURE;
     }
     
     /* create the socket */
     if ((sockfd = psock_socket(PSOCK_PF_INET,PSOCK_SOCK_DGRAM,PSOCK_IPPROTO_UDP))<0)
     {
-	eventlog(eventlog_level_error,"main","could not create UDP listen socket (psock_socket: %s)\n",strerror(psock_errno()));
+	eventlog(eventlog_level_error,__FUNCTION__,"could not create UDP listen socket (psock_socket: %s)\n",pstrerror(psock_errno()));
 	return STATUS_FAILURE;
     }
     
@@ -258,7 +259,7 @@ extern int main(int argc, char * argv[])
 	servaddr.sin_port = htons(prefs.port);
 	if (psock_bind(sockfd,(struct sockaddr *)&servaddr,sizeof(servaddr))<0)
 	{
-	    eventlog(eventlog_level_error,"main","could not bind to UDP port %hu (psock_bind: %s)\n",prefs.port,strerror(psock_errno()));
+	    eventlog(eventlog_level_error,__FUNCTION__,"could not bind to UDP port %hu (psock_bind: %s)\n",prefs.port,pstrerror(psock_errno()));
 	    return STATUS_FAILURE;
 	}
     }
@@ -284,7 +285,7 @@ static int server_process(int sockfd)
     
     if (!(serverlist_head = list_create()))
     {
-	eventlog(eventlog_level_error,"server_process","could not create server list");
+	eventlog(eventlog_level_error,__FUNCTION__,"could not create server list");
 	return -1;
     }
     
@@ -292,8 +293,6 @@ static int server_process(int sockfd)
     last = time(NULL) - prefs.update;
     for (;;)
     {
-	list_purge(serverlist_head);
-	
 	/* time to dump our list to disk and call the process command */
 	/* (I'm making the assumption that this won't take very long.) */
 	if (last+(signed)prefs.update<time(NULL))
@@ -302,7 +301,7 @@ static int server_process(int sockfd)
 	    
 	    if (!(outfile = fopen(prefs.outfile,"w")))
 	    {
-		eventlog(eventlog_level_error,"server_process","unable to open file \"%s\" for writing (fopen: %s)",prefs.outfile,strerror(errno));
+		eventlog(eventlog_level_error,__FUNCTION__,"unable to open file \"%s\" for writing (fopen: %s)",prefs.outfile,pstrerror(errno));
 		continue;
 	    }
 	    
@@ -312,8 +311,8 @@ static int server_process(int sockfd)
 		
 		if (server->updated+(signed)prefs.expire<last)
 		{
-		    list_remove_elem(serverlist_head,curr);
-		    free(server);
+		    list_remove_elem(serverlist_head,&curr);
+		    xfree(server);
 		}
 		else
 		{
@@ -360,7 +359,7 @@ static int server_process(int sockfd)
 		}
 	    }
             if (fclose(outfile)<0)
-                eventlog(eventlog_level_error,"main","could not close output file \"%s\" after writing (fclose: %s)",prefs.outfile,strerror(errno));
+                eventlog(eventlog_level_error,__FUNCTION__,"could not close output file \"%s\" after writing (fclose: %s)",prefs.outfile,pstrerror(errno));
 	    
 	    if (prefs.process[0]!='\0')
 		system(prefs.process);
@@ -379,7 +378,7 @@ static int server_process(int sockfd)
                 errno!=PSOCK_EINTR &&
 #endif
                 1)
-                eventlog(eventlog_level_error,"server_process","select failed (select: %s)",strerror(errno));
+                eventlog(eventlog_level_error,__FUNCTION__,"select failed (select: %s)",pstrerror(errno));
         case 0: /* timeout and no sockets ready */
             continue;
         }
@@ -428,8 +427,8 @@ static int server_process(int sockfd)
 			{
 			    if (ntohl(packet.flags)&TF_SHUTDOWN)
 			    {
-				list_remove_elem(serverlist_head,curr);
-				free(server);
+				list_remove_elem(serverlist_head,&curr);
+				xfree(server);
 			    }
 			    else
 			    {
@@ -444,19 +443,15 @@ static int server_process(int sockfd)
 		    /* Not found? Make a new slot */
 		    if (!(ntohl(packet.flags)&TF_SHUTDOWN) && !curr)
 		    {
-			if ((server = malloc(sizeof(t_server))))
-			{
-			    server->address = cliaddr.sin_addr;
-			    server->info = packet;
-			    server->updated = time(NULL);
-			    
-			    list_append_data(serverlist_head,server);
-			}
-			else
-			    eventlog(eventlog_level_error,"server_process","could not allocate memory for server");
+			server = xmalloc(sizeof(t_server));
+			server->address = cliaddr.sin_addr;
+			server->info = packet;
+			server->updated = time(NULL);
+
+			list_append_data(serverlist_head,server);
 		    }
 		    
-		    eventlog(eventlog_level_debug,"server_process",
+		    eventlog(eventlog_level_debug,__FUNCTION__,
 			     "Packet received from %s:"
 			     " packet_version=%u"
 			     " flags=0x%08lx"
