@@ -93,23 +93,21 @@
 #endif
 #include "compat/inet_ntoa.h"
 #include "compat/psock.h"
-
-#ifdef WIN32
-# include <io.h>
-# define F_OK 0
-#endif
-
+#include "compat/access.h"
+#include "compat/rename.h"
 #include "dbserver.h"
 #include "dbspacket.h"
 #include "d2ladder.h"
 #include "charlock.h"
 #include "prefs.h"
-#include "d2cs/xstring.h"
-#include "d2cs/d2cs_d2gs_character.h"
 #include "common/bn_type.h"
+#include "common/d2char_checksum.h"
+#include "common/xstring.h"
+#include "d2cs/d2cs_d2gs_character.h"
 #include "common/list.h"
 #include "common/eventlog.h"
 #include "common/addr.h"
+#include "common/xalloc.h"
 #include "common/setup_after.h"
 
 static unsigned int dbs_packet_savedata_charsave(t_d2dbs_connection* conn,char * AccountName,char * CharName,char * data,unsigned int datalen);
@@ -123,6 +121,9 @@ static unsigned int dbs_packet_charlock(t_d2dbs_connection* conn);
 static unsigned int dbs_packet_updateladder(t_d2dbs_connection* conn);
 static int dbs_verify_ipaddr(char const * addrlist,t_d2dbs_connection * c);
 
+static int dbs_packet_fix_charinfo(t_d2dbs_connection * conn,char * AccountName,char * CharName,char * charsave);
+static void dbs_packet_set_charinfo_level(char * CharName,char * charinfo);
+
 static unsigned int dbs_packet_savedata_charsave(t_d2dbs_connection* conn, char * AccountName,char * CharName,char * data,unsigned int datalen)
 {
 	char filename[MAX_PATH];
@@ -130,10 +131,23 @@ static unsigned int dbs_packet_savedata_charsave(t_d2dbs_connection* conn, char
 	char bakfile[MAX_PATH];
 	unsigned short curlen,readlen,leftlen,writelen;
 	FILE * fd;
+	int checksum_header;
+	int checksum_calc;
 	
 	strtolower(AccountName);
 	strtolower(CharName);
 
+	//check if checksum is ok
+	checksum_header = bn_int_get(&data[D2CHARSAVE_CHECKSUM_OFFSET]);
+	checksum_calc   = d2charsave_checksum(data,datalen,D2CHARSAVE_CHECKSUM_OFFSET);
+
+	if (checksum_header != checksum_calc)
+	{
+	  eventlog(eventlog_level_error,__FUNCTION__,"received (%x) and calculated(%x) checksum do not match - discarding charsave",checksum_header, checksum_calc);
+	  return 0;
+	}
+
+
 	sprintf(filename,"%s/.%s.tmp",d2dbs_prefs_get_charsave_dir(),CharName);
 	fd = fopen(filename, "wb");
 	if (!fd) {
@@ -148,7 +162,7 @@ static unsigned int dbs_packet_savedata_charsave(t_d2dbs_connection* conn, char
 		readlen=fwrite(data+curlen,1,writelen,fd);
 		if (readlen<=0) {
 			fclose(fd);
-			eventlog(eventlog_level_error,__FUNCTION__,"write() failed error : %s",strerror(errno));
+			eventlog(eventlog_level_error,__FUNCTION__,"write() failed error : %s",pstrerror(errno));
 			return 0;
 		}
 		curlen+=readlen;
@@ -158,26 +172,10 @@ static unsigned int dbs_packet_savedata_charsave(t_d2dbs_connection* conn, char
 
 	sprintf(bakfile,"%s/%s",prefs_get_charsave_bak_dir(),CharName);
 	sprintf(savefile,"%s/%s",d2dbs_prefs_get_charsave_dir(),CharName);
-#ifdef WIN32	
-	if (access(bakfile, 0) == 0) {
-		if (remove(bakfile)<0) {
-			eventlog(eventlog_level_error,__FUNCTION__,"could not delete backup charsave file \"%s\" (remove: %s)",bakfile);
-			return -1;
-		}
-	}
-#endif
-	if (rename(savefile, bakfile)==-1) {
+	if (p_rename(savefile, bakfile)==-1) {
 		eventlog(eventlog_level_warn,__FUNCTION__,"error rename %s to %s", savefile, bakfile);
 	}
-#ifdef WIN32
-	if (access(savefile, 0) == 0) {
-		if (remove(savefile)<0) {
-			eventlog(eventlog_level_error,__FUNCTION__,"could not delete charsave file \"%s\" (remove: %s)",savefile);
-			return -1;
-		}
-	}
-#endif
-	if (rename(filename, savefile)==-1) {
+	if (p_rename(filename, savefile)==-1) {
 		eventlog(eventlog_level_error,__FUNCTION__,"error rename %s to %s", filename, savefile);
 		return 0;
 	}
@@ -219,7 +217,7 @@ static unsigned int dbs_packet_savedata_charinfo(t_d2dbs_connection* conn,char *
 		readlen=fwrite(data+curlen,1,writelen,fd);
 		if (readlen<=0) {
 			fclose(fd);
-			eventlog(eventlog_level_error,__FUNCTION__,"write() failed error : %s",strerror(errno));
+			eventlog(eventlog_level_error,__FUNCTION__,"write() failed error : %s",pstrerror(errno));
 			return 0;
 		}
 		curlen+=readlen;
@@ -229,26 +227,10 @@ static unsigned int dbs_packet_savedata_charinfo(t_d2dbs_connection* conn,char *
 
 	sprintf(bakfile,"%s/%s/%s",prefs_get_charinfo_bak_dir(),AccountName,CharName);
 	sprintf(savefile,"%s/%s/%s",d2dbs_prefs_get_charinfo_dir(),AccountName,CharName);
-#ifdef WIN32
-	if (access(bakfile, 0) == 0) {
-		if (remove(bakfile)<0) {
-			eventlog(eventlog_level_error,__FUNCTION__,"could not delete backup charinfo file \"%s\" (remove: %s)",bakfile);
-			return -1;
-		}
-	}
-#endif
-	if (rename(savefile, bakfile)==-1) {
+	if (p_rename(savefile, bakfile)==-1) {
 		eventlog(eventlog_level_info,__FUNCTION__,"error rename %s to %s", savefile, bakfile);
 	}
-#ifdef WIN32
-	if (access(savefile, 0) == 0) {
-		if (remove(savefile)<0) {
-			eventlog(eventlog_level_error,__FUNCTION__,"could not delete charinfo file \"%s\" (remove: %s)",savefile);
-			return -1;
-		}
-	}
-#endif
-	if (rename(filename, savefile)==-1) {
+	if (p_rename(filename, savefile)==-1) {
 		eventlog(eventlog_level_error,__FUNCTION__,"error rename %s to %s", filename, savefile);
 		return 0;
 	}
@@ -300,7 +282,7 @@ static unsigned int dbs_packet_getdata_charsave(t_d2dbs_connection* conn,char *
 		readlen=fread(data+curlen,1,writelen,fd);
 		if (readlen<=0) {
 			fclose(fd);
-			eventlog(eventlog_level_error,__FUNCTION__,"read() failed error : %s",strerror(errno));
+			eventlog(eventlog_level_error,__FUNCTION__,"read() failed error : %s",pstrerror(errno));
 			return 0;
 		}
 		leftlen-=readlen;
@@ -352,7 +334,7 @@ static unsigned int dbs_packet_getdata_charinfo(t_d2dbs_connection* conn,char *
 	    if (readlen<=0)
 		{
 			fclose(fd);
-			eventlog(eventlog_level_error,__FUNCTION__,"read() failed error : %s",strerror(errno));
+			eventlog(eventlog_level_error,__FUNCTION__,"read() failed error : %s",pstrerror(errno));
 			return 0;
 		}
 		leftlen-=readlen;
@@ -369,9 +351,9 @@ static unsigned int dbs_packet_savedata(t_d2dbs_connection * conn)
 	unsigned short      datatype;
 	unsigned short      datalen; 
 	unsigned int        result; 
-	char AccountName[MAX_NAME_LEN+16];
-	char CharName[MAX_NAME_LEN+16];
-	char RealmName[MAX_NAME_LEN+16];
+	char AccountName[MAX_ACCTNAME_LEN];
+	char CharName[MAX_CHARNAME_LEN];
+	char RealmName[MAX_REALMNAME_LEN];
 	t_d2gs_d2dbs_save_data_request	* savecom; 
 	t_d2dbs_d2gs_save_data_reply	* saveret; 
 	char * readpos;
@@ -383,14 +365,26 @@ static unsigned int dbs_packet_savedata(t_d2dbs_connection * conn)
 	datalen=bn_short_get(savecom->datalen);
 
 	readpos+=sizeof(*savecom);
-	strncpy(AccountName,readpos,MAX_NAME_LEN);
-	AccountName[MAX_NAME_LEN]=0;
+	strncpy(AccountName,readpos,MAX_ACCTNAME_LEN);
+	if (AccountName[MAX_ACCTNAME_LEN-1]!=0)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"max acccount name length exceeded");
+	    return -1;
+	}
 	readpos+=strlen(AccountName)+1;
-	strncpy(CharName,readpos,MAX_NAME_LEN);
-	CharName[MAX_NAME_LEN]=0;
+	strncpy(CharName,readpos,MAX_CHARNAME_LEN);
+	if (CharName[MAX_CHARNAME_LEN-1]!=0)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"max char name length exceeded");
+	    return -1;
+	}
 	readpos+=strlen(CharName)+1;
-	strncpy(RealmName,readpos,MAX_NAME_LEN);
-	RealmName[MAX_NAME_LEN]=0;
+	strncpy(RealmName,readpos,MAX_REALMNAME_LEN);
+	if (RealmName[MAX_REALMNAME_LEN-1]!=0)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"max realm name length exceeded");
+	    return -1;
+	}
 	readpos+=strlen(RealmName)+1;
 
 	if (readpos+datalen!=conn->ReadBuf+bn_short_get(savecom->h.size)) {
@@ -399,13 +393,16 @@ static unsigned int dbs_packet_savedata(t_d2dbs_connection * conn)
 	}
 
 	if (datatype==D2GS_DATA_CHARSAVE) {
-		if (dbs_packet_savedata_charsave(conn,AccountName,CharName,readpos,datalen)>0) {
+		if (dbs_packet_savedata_charsave(conn,AccountName,CharName,readpos,datalen)>0 &&
+		    dbs_packet_fix_charinfo(conn,AccountName,CharName,readpos)) {
 			result=D2DBS_SAVE_DATA_SUCCESS;
 		} else {
 			datalen=0;
 			result=D2DBS_SAVE_DATA_FAILED    ;
 		}
 	} else if (datatype==D2GS_DATA_PORTRAIT) {
+		/* if level is > 255 , sets level to 255 */
+		dbs_packet_set_charinfo_level(CharName,readpos);
 		if (dbs_packet_savedata_charinfo(conn,AccountName,CharName,readpos,datalen)>0) {
 			result=D2DBS_SAVE_DATA_SUCCESS;
 		} else {
@@ -426,7 +423,7 @@ static unsigned int dbs_packet_savedata(t_d2dbs_connection * conn)
 	bn_short_set(&saveret->datatype,bn_short_get(savecom->datatype));
 	bn_int_set(&saveret->result,result);
 	writepos+=sizeof(*saveret);
-	strncpy(writepos,CharName,MAX_NAME_LEN);
+	strncpy(writepos,CharName,MAX_CHARNAME_LEN);
 	conn->nCharsInWriteBuffer += writelen;
 	return 1;
 }
@@ -443,9 +440,9 @@ static unsigned int dbs_packet_getdata(t_d2dbs_connection * conn)
 	unsigned short	datatype;
 	unsigned short	datalen; 
 	unsigned int	result; 
-	char		AccountName[MAX_NAME_LEN+16];
-	char		CharName[MAX_NAME_LEN+16];
-	char		RealmName[MAX_NAME_LEN+16];
+	char		AccountName[MAX_ACCTNAME_LEN];
+	char		CharName[MAX_CHARNAME_LEN];
+	char		RealmName[MAX_REALMNAME_LEN];
 	t_d2gs_d2dbs_get_data_request	* getcom; 
 	t_d2dbs_d2gs_get_data_reply	* getret; 
 	char		* readpos;
@@ -460,14 +457,26 @@ static unsigned int dbs_packet_getdata(t_d2dbs_connection * conn)
 	datatype=bn_short_get(getcom->datatype);
 	
 	readpos+=sizeof(*getcom);
-	strncpy(AccountName,readpos,MAX_NAME_LEN);
-	AccountName[MAX_NAME_LEN]=0;
+	strncpy(AccountName,readpos,MAX_ACCTNAME_LEN);
+	if (AccountName[MAX_ACCTNAME_LEN-1]!=0)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"max account name length exceeded");
+	    return -1;
+	}
 	readpos+=strlen(AccountName)+1;
-	strncpy(CharName,readpos,MAX_NAME_LEN);
-	CharName[MAX_NAME_LEN]=0;
+	strncpy(CharName,readpos,MAX_CHARNAME_LEN);
+	if (CharName[MAX_CHARNAME_LEN-1]!=0)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"max char name length exceeded");
+	    return -1;
+	}
 	readpos+=strlen(CharName)+1;
-	strncpy(RealmName,readpos,MAX_NAME_LEN);
-	RealmName[MAX_NAME_LEN]=0;
+	strncpy(RealmName,readpos,MAX_REALMNAME_LEN);
+	if (RealmName[MAX_REALMNAME_LEN-1]!=0)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"max realm name length exceeded");
+	    return -1;
+	}
 	readpos+=strlen(RealmName)+1;
 
 	if (readpos != conn->ReadBuf+bn_short_get(getcom->h.size)) {
@@ -547,7 +556,7 @@ static unsigned int dbs_packet_getdata(t_d2dbs_connection * conn)
 	bn_int_set(&getret->result,result);
 	bn_short_set(&getret->datalen,datalen);
 	writepos+=sizeof(*getret);
-	strncpy(writepos,CharName,MAX_NAME_LEN);
+	strncpy(writepos,CharName,MAX_CHARNAME_LEN);
 	writepos+=strlen(CharName)+1;
 	if (datalen) memcpy(writepos,databuf,datalen);
 	conn->nCharsInWriteBuffer += writelen;
@@ -556,8 +565,8 @@ static unsigned int dbs_packet_getdata(t_d2dbs_connection * conn)
 
 static unsigned int dbs_packet_updateladder(t_d2dbs_connection * conn)
 {
-	char CharName[MAX_NAME_LEN+16];
-	char RealmName[MAX_NAME_LEN+16];
+	char CharName[MAX_CHARNAME_LEN];
+	char RealmName[MAX_REALMNAME_LEN];
 	t_d2gs_d2dbs_update_ladder	* updateladder; 
 	char * readpos;
 	t_d2ladder_info			charladderinfo;
@@ -566,11 +575,19 @@ static unsigned int dbs_packet_updateladder(t_d2dbs_connection * conn)
 	updateladder=(t_d2gs_d2dbs_update_ladder *)readpos;
 	
 	readpos+=sizeof(*updateladder);
-	strncpy(CharName,readpos,MAX_NAME_LEN);
-	CharName[MAX_NAME_LEN]=0;
+	strncpy(CharName,readpos,MAX_CHARNAME_LEN);
+	if (CharName[MAX_CHARNAME_LEN-1]!=0)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"max char name length exceeded");
+	    return -1;
+	}
 	readpos+=strlen(CharName)+1;
-	strncpy(RealmName,readpos,MAX_NAME_LEN);
-	RealmName[MAX_NAME_LEN]=0;
+	strncpy(RealmName,readpos,MAX_REALMNAME_LEN);
+	if (RealmName[MAX_REALMNAME_LEN-1]!=0)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"max realm name length exceeded");
+	    return -1;
+	}
 	readpos+=strlen(RealmName)+1;
 	if (readpos != conn->ReadBuf+bn_short_get(updateladder->h.size)) {
 		eventlog(eventlog_level_error,__FUNCTION__,"request packet size error");
@@ -589,9 +606,9 @@ static unsigned int dbs_packet_updateladder(t_d2dbs_connection * conn)
 
 static unsigned int dbs_packet_charlock(t_d2dbs_connection * conn)
 {
-	char CharName[MAX_NAME_LEN+16];
-	char AccountName[MAX_NAME_LEN+16];
-	char RealmName[MAX_NAME_LEN+16];
+	char CharName[MAX_CHARNAME_LEN];
+	char AccountName[MAX_ACCTNAME_LEN];
+	char RealmName[MAX_REALMNAME_LEN];
 	t_d2gs_d2dbs_char_lock * charlock; 
 	char * readpos;
 
@@ -599,14 +616,26 @@ static unsigned int dbs_packet_charlock(t_d2dbs_connection * conn)
 	charlock=(t_d2gs_d2dbs_char_lock*)readpos;
 	
 	readpos+=sizeof(*charlock);
-	strncpy(AccountName,readpos,MAX_NAME_LEN);
-	AccountName[MAX_NAME_LEN]=0;
+	strncpy(AccountName,readpos,MAX_ACCTNAME_LEN);
+	if (AccountName[MAX_ACCTNAME_LEN-1]!=0)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"max account name length exceeded");
+	    return -1;
+	}
 	readpos+=strlen(AccountName)+1;
-	strncpy(CharName,readpos,MAX_NAME_LEN);
-	CharName[MAX_NAME_LEN]=0;
+	strncpy(CharName,readpos,MAX_CHARNAME_LEN);
+	if (CharName[MAX_CHARNAME_LEN-1]!=0)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"max char name length exceeded");
+	    return -1;
+	}
 	readpos+=strlen(CharName)+1;
-	strncpy(RealmName,readpos,MAX_NAME_LEN);
-	RealmName[MAX_NAME_LEN]=0;
+	strncpy(RealmName,readpos,MAX_REALMNAME_LEN);
+	if (RealmName[MAX_REALMNAME_LEN-1]!=0)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"max realm name length exceeded");
+	    return -1;
+	}
 	readpos+=strlen(RealmName)+1;
 
 	if (readpos != conn->ReadBuf+ bn_short_get(charlock->h.size)) {
@@ -720,7 +749,7 @@ static int dbs_verify_ipaddr(char const * addrlist,t_d2dbs_connection * c)
 
 	in.s_addr=htonl(c->ipaddr);
 	ipaddr=inet_ntoa(in);
-	if (!(adlist = strdup(addrlist))) return -1;
+	adlist = xstrdup(addrlist);
 	temp=adlist;
 	valid=0;
 	while ((s=strsep(&temp, ","))) {
@@ -732,7 +761,7 @@ static int dbs_verify_ipaddr(char const * addrlist,t_d2dbs_connection * c)
 			break;
 		}
 	}
-	free(adlist);
+	xfree(adlist);
 	if (valid) {
 		eventlog(eventlog_level_info,__FUNCTION__,"ip address %s is valid",ipaddr);
 		LIST_TRAVERSE(dbs_server_connection_list,elem)
@@ -741,7 +770,7 @@ static int dbs_verify_ipaddr(char const * addrlist,t_d2dbs_connection * c)
 			if (tempc !=c && tempc->ipaddr==c->ipaddr) {
 				eventlog(eventlog_level_info,__FUNCTION__,"destroying previous connection %d",tempc->serverid);
 				dbs_server_shutdown_connection(tempc);
-				list_remove_elem(dbs_server_connection_list,elem);
+				list_remove_elem(dbs_server_connection_list,&elem);
 			}
 		}
 		c->verified = 1;
@@ -767,7 +796,7 @@ int dbs_check_timeout(void)
 		if (now-tempc->last_active>timeout) {
 			eventlog(eventlog_level_debug,__FUNCTION__,"connection %d timed out",tempc->serverid);
 			dbs_server_shutdown_connection(tempc);
-			list_remove_elem(dbs_server_connection_list,elem);
+			list_remove_elem(dbs_server_connection_list,&elem);
 			continue;
 		}
 	}
@@ -799,3 +828,96 @@ int dbs_keepalive(void)
 	}
 	return 0;
 }
+
+/*************************************************************************************/
+#define CHARINFO_SIZE			0xC0
+#define CHARINFO_PORTRAIT_LEVEL_OFFSET	0x89
+#define CHARINFO_PORTRAIT_STATUS_OFFSET	0x8A
+#define CHARINFO_SUMMARY_LEVEL_OFFSET	0xB8
+#define CHARINFO_SUMMARY_STATUS_OFFSET	0xB4
+#define CHARINFO_PORTRAIT_GFX_OFFSET	0x72
+#define CHARINFO_PORTRAIT_COLOR_OFFSET	0x7E
+
+#define CHARSAVE_LEVEL_OFFSET		0x2B
+#define CHARSAVE_STATUS_OFFSET		0x24
+#define CHARSAVE_GFX_OFFSET		0x88
+#define CHARSAVE_COLOR_OFFSET		0x98
+
+#define charstatus_to_portstatus(status) ((((status & 0xFF00) << 1) | (status & 0x00FF)) | 0x8080)
+#define portstatus_to_charstatus(status) (((status & 0x7F00) >> 1) | (status & 0x007F))
+
+static void dbs_packet_set_charinfo_level(char * CharName,char * charinfo)
+{
+    if (prefs_get_difficulty_hack()) { /* difficulty hack enabled */
+	unsigned int	level = bn_int_get(&charinfo[CHARINFO_SUMMARY_LEVEL_OFFSET]);
+	unsigned int	plevel = bn_byte_get(&charinfo[CHARINFO_PORTRAIT_LEVEL_OFFSET]);
+	
+	/* levels 257 thru 355 */
+	if (level != plevel) {
+	    eventlog(eventlog_level_info,__FUNCTION__,"level mis-match for %s ( %u != %u ) setting to 255",CharName,level,plevel);
+	    bn_byte_set((bn_byte *)&charinfo[CHARINFO_PORTRAIT_LEVEL_OFFSET],255);
+	    bn_int_set((bn_int *)&charinfo[CHARINFO_SUMMARY_LEVEL_OFFSET],255);
+	}
+    }
+}
+
+static int dbs_packet_fix_charinfo(t_d2dbs_connection * conn,char * AccountName,char * CharName,char * charsave)
+{
+    if (prefs_get_difficulty_hack()) {
+	unsigned char	charinfo[CHARINFO_SIZE];
+	unsigned int	level = bn_byte_get(&charsave[CHARSAVE_LEVEL_OFFSET]);
+	unsigned short	status = bn_short_get(&charsave[CHARSAVE_STATUS_OFFSET]);
+	unsigned short	pstatus = charstatus_to_portstatus(status);
+	int		i;
+	
+	/*
+	 * charinfo is only updated from level 1 to 99 (d2gs issue)
+	 * from 100 to 256 d2gs does not send it
+	 * when value rolls over (level 256 = 0)
+	 * and charactar reaches level 257 (rolled over to level 1)
+	 * d2gs starts sending it agian until level 356 (rolled over to 100)
+	 * is reached agian. etc. etc. etc.
+	 */
+	if (level == 0) /* level 256, 512, 768, etc */
+	    level = 255;
+	
+	if (level < 100)
+	    return 1; /* d2gs will send charinfo - level will be set to 255 at that time if needed */
+	
+	eventlog(eventlog_level_info,__FUNCTION__,"level %u > 99 for %s",level,CharName);
+	
+	if(!(dbs_packet_getdata_charinfo(conn,AccountName,CharName,charinfo,CHARINFO_SIZE))) {
+	    eventlog(eventlog_level_error,__FUNCTION__,"unable to get charinfo for %s",CharName);
+	    return 0;
+	}
+	
+	/* if level in charinfo file is already set to 255,
+	 * then is must have been set when d2gs sent the charinfo 
+	 * and got a level mis-match (levels 257 - 355)
+	 * or level is actually 255. In eather case we set to 255
+	 * this should work for any level mod
+	 */
+	if (bn_byte_get(&charinfo[CHARINFO_PORTRAIT_LEVEL_OFFSET]) == 255)
+	    level = 255;
+	
+	eventlog(eventlog_level_info,__FUNCTION__,"updating charinfo for %s -> level = %u , status = 0x%04X , pstatus = 0x%04X",CharName,level,status,pstatus);
+	bn_byte_set((bn_byte *)&charinfo[CHARINFO_PORTRAIT_LEVEL_OFFSET],level);
+	bn_int_set((bn_int *)&charinfo[CHARINFO_SUMMARY_LEVEL_OFFSET],level);
+	bn_short_set((bn_short *)&charinfo[CHARINFO_PORTRAIT_STATUS_OFFSET],pstatus);
+	bn_int_set((bn_int *)&charinfo[CHARINFO_SUMMARY_STATUS_OFFSET],status);
+	
+	for (i=0;i<11;i++) {
+	    bn_byte_set((bn_byte *)&charinfo[CHARINFO_PORTRAIT_GFX_OFFSET+i],bn_byte_get(&charsave[CHARSAVE_GFX_OFFSET+i]));
+	    bn_byte_set((bn_byte *)&charinfo[CHARINFO_PORTRAIT_COLOR_OFFSET+i],bn_byte_get(&charsave[CHARSAVE_GFX_OFFSET+i]));
+	}
+	
+	if (!(dbs_packet_savedata_charinfo(conn,AccountName,CharName,charinfo,CHARINFO_SIZE))) {
+	    eventlog(eventlog_level_error,__FUNCTION__,"unable to save charinfo for %s",CharName);
+	    return 0;
+	}
+	
+	return 1; /* charinfo updated */
+    }
+    
+    return 1; /* difficulty hack not enabled */
+}
